C
C     AUTOSLIT.INC
C     Include file containing slit, spectrograph and object definitions.
C
C     Original version by Doug Clowe, January 1993
C
C     Modifications:  
C
C     Jan-1994 - Minimum slit length decreased from 15 to 5 mm.  [JLC]
C     Mov-1996 - Additional slit types added for version 3.0.    [JLC]
C     Mar-1997 - Added parameter values with default settings.   [PLS]
C     Mar-1997 - Mathematical defines moved to math.inc.         [PLS]
C     Aug-1997 - Modified pickoff mirror location.               [JLC]
C     Aug-1997 - Increased DEF_WSLIT from 0.7" to 1.0".          [JLC]
C     Aug-1997 - Decreased NORM_SLEN from 8" to 5".              [JLC]
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.        [PLS]
C     Aug-1997 - Changed SLIT_SEP from integer to real.          [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.             [PLS]
C     Oct-1997 - Added GUIDE_OBJ variable.                       [PLS]
C     Oct-1997 - Changed EPOCHS_CENTER to EQUINOX_CENTER.        [PLS]
C     Oct-1997 - Changed DEF_INPUT to 'autoslit.in'.             [PLS]
C     Oct-1997 - Added DEF_OBJ as 'autoslit.obj'.                [PLS]

      REAL*4 VERSION
      PARAMETER (VERSION=3.05)                   ! Autoslit version number

C     Default values of autoslit parameters
      CHARACTER*64 DEF_INPUT,DEF_OBJ
      CHARACTER*64 DEF_OUTPUT,DEF_OUT2,DEF_OUT3
      PARAMETER (DEF_INPUT='autoslit.in',	! input parameter file
     :           DEF_OBJ='autoslit.obj',        ! input file of objects
     :           DEF_OUTPUT='autoslit.file1',	! primary output file
     :           DEF_OUT2='autoslit.file2',	! output file of object stage coords
     :           DEF_OUT3='autoslit.file3')	! output file for punch/mill machine
      INTEGER*4 DEF_PRIORITY,DEF_RANK_TOP,DEF_RANK_BOT
      PARAMETER (DEF_PRIORITY=700,		! priority for plot symbol change
     :           DEF_RANK_TOP=1,		! rank of top half of slit (1 or 2)
     :           DEF_RANK_BOT=1)		! rank of bottom half of slit (1 or 2)
      REAL*4     DEF_SLIT_WID,DEF_SLIT_LEN,DEF_SLIT_SEP
      PARAMETER (DEF_SLIT_WID=1.0,              ! normal width, length,
     :           DEF_SLIT_LEN=5.0,              ! and separation of
     :           DEF_SLIT_SEP=2.757)            ! slits [arc-sec]
      REAL*4     ENG_SLIT_WID,ENG_SLIT_LEN,ENG_SLIT_SEP
      PARAMETER (ENG_SLIT_WID=1.0,              ! engineering width, length,
     :           ENG_SLIT_LEN=1.0,              ! and separation of
     :           ENG_SLIT_SEP=2.757)            ! slits [arc-sec]
      REAL*8    DEF_TEMP,DEF_PRES,DEF_WAVE,DEF_LAT,DEF_POS_ANGLE,DEF_H
      PARAMETER (DEF_TEMP=0.0D0,		! temperature
     :           DEF_PRES=486.0D0,		! air pressure at Keck
     :           DEF_WAVE=6000.0D0,		! wavelength for refraction
     :           DEF_LAT=19.828D0,		! latitude of Keck
     :           DEF_POS_ANGLE=0.0D0,		! position angle
     :           DEF_H=0.0D0)			! hour angle
      LOGICAL   DEF_MIRROR_USE,DEF_ENG_TEST,DEF_INTERACTIVE,DEF_VERBOSE
      PARAMETER (DEF_MIRROR_USE=.FALSE.,	! use small pickoff mirror?
     :           DEF_ENG_TEST=.FALSE.,		! is this an engineering test?
     :           DEF_INTERACTIVE=.TRUE.,	! interactive slit selection?
     :           DEF_VERBOSE=.FALSE.)		! verbose output?

      CHARACTER SUFFIX2*10,SUFFIX3*10,SUFFIX4*10
      PARAMETER (SUFFIX2='file2',		! suffix for outfile2
     :           SUFFIX3='file3',		! suffix for outfile3
     :           SUFFIX4='ps')                  ! suffix for hardcopy

      CHARACTER DEF_DEVICE*32,DEF_HDEVICE*32,NULL_DEVICE*32
      PARAMETER (DEF_DEVICE='/xwin',            ! interactive plotting device
     :           DEF_HDEVICE='/vps',            ! hardcopy plotting device
     :           NULL_DEVICE='/null')           ! null plotting device

C     Object definitions:
      INTEGER*4 MAXOBJ,NPTS,NUSE
      PARAMETER (MAXOBJ=3000)
      REAL*8    RAS(MAXOBJ),DECS(MAXOBJ)
      REAL*8    RAPREC(MAXOBJ),DECPREC(MAXOBJ)
      REAL*8    RAREFR(MAXOBJ),DECREFR(MAXOBJ)
      REAL*4    DRAS(MAXOBJ),DDECS(MAXOBJ)                
      REAL*4    EPOCHS(MAXOBJ),EQUINOXS(MAXOBJ),MAGS(MAXOBJ)
      INTEGER*4 PRIORS(MAXOBJ),INDX(MAXOBJ)
      CHARACTER LINE(MAXOBJ)*254,ONAMES(MAXOBJ)*40
      COMMON /OBJECTS/ 
     :      RAS,DECS,RAPREC,DECPREC,RAREFR,DECREFR,
     :      DRAS,DDECS,
     :      EPOCHS,EQUINOXS,MAGS,
     :      PRIORS,INDX,
     :      LINE,ONAMES,
     :	    NPTS,NUSE

C     More object definitions:
      REAL*4    OBJ_T_X(MAXOBJ),OBJ_T_Y(MAXOBJ)
      LOGICAL   OBJ_HIT(MAXOBJ),OBJ_OUT(MAXOBJ)
      INTEGER*4 OBJ_SLIT(MAXOBJ),OBJ_STYPE(MAXOBJ),OBJ_SIND(MAXOBJ)
      COMMON    /SLITS/
     :          OBJ_T_X,OBJ_T_Y,
     :          OBJ_HIT,OBJ_OUT,
     :          OBJ_SLIT,OBJ_STYPE,OBJ_SIND

C     Slit types
      INTEGER*4 STD_SLIT,TILT_SLIT,ARC_SLIT,LINE_SLIT,BOX_SLIT
      PARAMETER (STD_SLIT =0,
     :           TILT_SLIT=1,
     :           ARC_SLIT =2,
     :           LINE_SLIT=3,
     :           BOX_SLIT =4)

C     Definitions for tilted slits
      INTEGER*4 MAXTILTS
      PARAMETER (MAXTILTS=30)
      REAL*4    TILT(MAXTILTS),TILT_MIN_UP(MAXTILTS)
      REAL*4    TILT_MIN_DOWN(MAXTILTS)
      INTEGER*4 TILT_OBJ(MAXTILTS)
      CHARACTER*40 TILT_OBJ_NAME(MAXTILTS)
      COMMON    /TILTS/ TILT,TILT_MIN_UP,TILT_MIN_DOWN,
     :                  TILT_OBJ,TILT_OBJ_NAME
      INTEGER*4 TILT_PRIOR
      PARAMETER (TILT_PRIOR=2100)

C     Definitions for arcs
      INTEGER*4 MAXARCS
      PARAMETER (MAXARCS=10)
      CHARACTER*40 ARC_NAME(MAXARCS)
      REAL*8    ARC_RA(MAXARCS),ARC_DEC(MAXARCS)
      REAL*4    ARC_RADIUS(MAXARCS),ARC_EPOCH(MAXARCS)
      REAL*4    ARC_MIN_UP(MAXARCS),ARC_MIN_DOWN(MAXARCS)
      INTEGER*4 ARC_OBJ(MAXARCS)
      COMMON    /ARCS/
     :          ARC_NAME,ARC_RA,ARC_DEC,ARC_EPOCH,
     :          ARC_RADIUS,ARC_MIN_UP,ARC_MIN_DOWN,ARC_OBJ
      INTEGER*4 ARC_PRIOR
      PARAMETER (ARC_PRIOR=2200)
      INTEGER*4 MAX_ARC_LEN
      PARAMETER (MAX_ARC_LEN=400)

C     Definitions for line segment slits
      INTEGER*4 MAXLINES
      PARAMETER (MAXLINES=30)
      CHARACTER*40 LINE_NAME(MAXLINES)
      INTEGER*4	LINE_INIT(MAXLINES),LINE_NUM(MAXLINES)
      REAL*8    LINE_RA(MAXLINES),LINE_DEC(MAXLINES)
      REAL*4    LINE_EPOCH(MAXLINES)
      INTEGER*4 LINE_OBJ(MAXLINES)
      INTEGER*4 NLINES                  ! total number of line segment pieces
      COMMON    /LINES/
     :          LINE_NAME,LINE_INIT,LINE_NUM,
     :          LINE_RA,LINE_DEC,LINE_EPOCH,LINE_OBJ,NLINES
      INTEGER*4 LINE_PRIOR
      PARAMETER (LINE_PRIOR=2300)

C     Definitions for boxes
      INTEGER*4 MAXBOXES
      PARAMETER (MAXBOXES=10)
      CHARACTER*40 BOX_OBJ_NAME(MAXBOXES)
      REAL*4    BOX_SIZE(MAXBOXES)
      INTEGER*4 BOX_OBJ(MAXBOXES)
      COMMON    /BOXES/
     :          BOX_OBJ_NAME,BOX_SIZE,BOX_OBJ
      INTEGER*4 BOX_PRIOR
      PARAMETER (BOX_PRIOR=2400)
      REAL*4    DEF_BOX_SIZE
      PARAMETER (DEF_BOX_SIZE=4.0)

C     Definitions for guide stars
      CHARACTER*40 GUIDE_STAR_NAME
      INTEGER*4 GUIDE_OBJ
      COMMON    /GUIDES/
     :          GUIDE_STAR_NAME,GUIDE_OBJ

C     Numbers of special objects: total numbers and assigned numbers
      INTEGER*4  NBOX,NARCS,NSEGS,NTILT,NSPECIAL
      INTEGER*4  MBOX,MARCS,MSEGS,MTILT,MSPECIAL
      COMMON /SPECIAL/
     :       NBOX,NARCS,NSEGS,NTILT,NSPECIAL,
     :       MBOX,MARCS,MSEGS,MTILT,MSPECIAL

C     Guider constants
      REAL*4    GXMIN,GXMAX,GYMIN,GYMAX,G_X,G_Y
      PARAMETER (GXMIN = 246.2,        ! pickoff mirror location [mm]
     :           GXMAX = 271.2,
     :           GYMIN = -30.0,
     :           GYMAX = 1.07,
     :           G_X   = 258.7,        ! center of pickoff mirror [mm]
     :           G_Y   = -16.0)
C    :           GYMIN = -23.93,
C    :           G_Y   = -11.43)

C     Bar constants
      REAL*4    BYMIN,BYMAX
      PARAMETER (BYMIN = -3.5,         ! location of central bar in slitmask
     :           BYMAX = +3.5)

C     Center of fields: original, after precession, and after refraction
      REAL*8    DRA0,DDEC0,DRA0PREC,DDEC0PREC,DRA0REFR,DDEC0REFR
      REAL*4    EQUINOX_CENTER
      COMMON    /CENTERS/ DRA0,DDEC0,DRA0PREC,DDEC0PREC,
     :                    DRA0REFR,DDEC0REFR,EQUINOX_CENTER

C     Spectrograph:
      INTEGER*4 NORM_SLITS,ENG_SLITS
      REAL*4    PUNCH_LEN,CCD_SIDE,CCD_SCALE,
     :          XS_SLIT,YS_SLIT,
     :          X_CENTER,Y_CENTER,
     :		SCALE,MASK_ANGLE,MASK_BEND
      PARAMETER (
     :           NORM_SLITS = 100,          ! number of slits for normal use
     :           ENG_SLITS  = 500,          ! number of slits for engineering
     :		 PUNCH_LEN  = 3.0,          ! length of punch used [not used]
     :           CCD_SIDE   = 2048.0,       ! size of ccd in pixels
     :           CCD_SCALE  = 0.15767,      ! mm per pixel of ccd at slitmask
     :           XS_SLIT    = 215.9,        ! length of the x side of the slitmask
     :           YS_SLIT    = 335.6,        ! length of the y side of the slitmask
     :           X_CENTER   = 305.0,        ! center of mask in x coordinates
     :           Y_CENTER   = 0.0,          ! center of mask in y coordiantes
     :		 SCALE      = 0.7253,       ! scale of mask in mm/arcsec
     :		 MASK_ANGLE = 8.06,	    ! angle of mask in focal plane in deg
     :		 MASK_BEND  = 1.94)         ! angle of bend in mask in deg

C     More spectrograph:
      REAL*4    XSIDE,YSIDE
      COMMON /SIDES/ XSIDE,YSIDE

C     Notches in slitmask:
      REAL*4    NOTCH1X,NOTCH1Y,NOTCH2X,NOTCH2Y
      PARAMETER (
     :           NOTCH1X = 172.67,
     :           NOTCH1Y = 152.40,
     :           NOTCH2X = 350.47,
     :           NOTCH2Y = 177.80)

C     Slit definitions:
      INTEGER*4 MAXSLITS,NSLITS
      REAL*4    SLIT_MIN_Y(ENG_SLITS), SLIT_MAX_Y(ENG_SLITS),
     :          SLIT_X(ENG_SLITS)
      INTEGER*4 SLIT_OBJ(ENG_SLITS)
      COMMON    /SLITS/
     :          MAXSLITS,NSLITS,
     :		SLIT_MIN_Y, SLIT_MAX_Y,
     :		SLIT_X,
     :		SLIT_OBJ

C     Marker definitions for GOTO command:
      LOGICAL   MARKER
      REAL*4    MARKX, MARKY, MARKCH
      INTEGER*4 MARKCI, MARKPT
      COMMON    /MARKS/
     :          MARKER,
     :          MARKX, MARKY
      PARAMETER (MARKCH=4.0,     ! Marker PGPLOT scaling
     :           MARKCI=2,	 ! Marker PGPLOT color index
     :           MARKPT=2)	 ! Marker PGPLOT point type

C     Output file string delimiters
      CHARACTER*1 STDL,LDL1,RDL1,LDL2,RDL2,LDL3,RDL3
      PARAMETER   (STDL='',	 ! delimiter for standard output
     :             LDL1='#',	 ! left delimiter for output file #1
     :             RDL1='#',	 ! right delimiter for output file #1
     :             LDL2=' ',	 ! left delimiter for output file #2
     :             RDL2=' ',	 ! right delimiter for output file #2
     :             LDL3='(',	 ! left delimiter for output file #3
     :             RDL3=')')	 ! right delimiter for output file #3

C     Mill machine output file characteristics
      REAL*4      X_MILL, Y_MILL, XOFF_MILL, YOFF_MILL, R_GUIDER
      INTEGER*4   CLOCKWISE_MILL
      CHARACTER*6 MILL_LABEL
      PARAMETER   (X_MILL = 177.8,	! X center in mill coord system
     :             Y_MILL = 132.3,	! Y center in mill coord system
C    :             XOFF_MILL = 10.0,	! X offset of mill coord system
C    :             YOFF_MILL = 24.5,	! X offset of mill coord system
     :             XOFF_MILL = 0.0,	! X offset of mill coord system
     :             YOFF_MILL = 0.0,	! X offset of mill coord system
     :             CLOCKWISE_MILL = -1, ! [-1|+1] for [CCW|CW] slit milling
     :             R_GUIDER = 5.0,	! radius of circular guider marker [mm]
     :             MILL_LABEL='newrow')

C     Miscellaneous defines
      INTEGER*4 STDOUT
      PARAMETER (STDOUT=6)

      REAL*4    DEFAULT          ! defaulted parameter value;
      PARAMETER (DEFAULT=0.0)    ! must match value in get_key.c

      INTEGER   UNDEFINED        ! undefined value;
      PARAMETER (UNDEFINED=-1)   ! must be < 0
