            program check_mask3
C
C
C         This program takes the slitmask machine output file from
C         Autoslit3 (where positinos are given in mm, and 1 arc-sec=
C         0.7253 mm) and produces a full size image of the multi-slit
C         mask.  The output is on 2 sheets of 8.5x11 inch paper, the
C         first sheet is the top of the mask, the second is the bottom.
C         The use of the Keck slitmask mill is assumed.
C
C         These output sheets can be used to check that the punching
C         machine produced a valid, correct slit-mask.
C
C         To print the output, type "lp -Pabc pgplot.ps", where abc is
C         the name of the Postscript printer.
C
C         Coded by J.Cohen, Oct. 1994, after a suggestion by Dru Phillips
C         Modified from the original check_mask.f for the Keck mill by
C         J.Cohen, July 1997.
C  
C         This program requires the PGPLOT subroutine library.
C
C         Mill parameter definitions:
C         nslit_max   maximum number of slits in a single slitmask
C         nmax        maximum number of points defining a single slit
C         nslit       number of slits actually on the mask
C         number(nslit)     number of (x,y) pairs in the nth slit
C         xoffset, yoffset      the coordinates of the corner
C                   marked 0,0 in the sketch of the mill axes below
C
C
C                                     X ---->
C                         -------------------------------
C                        |    --            xxxxxxxx.xxx |
C                        |                               |
C                        o       ----            ------  |
C                      Y |               --              |
C                        |                               |
C                      | |    As viewed from the stars   |
C                      | |                               |
C                      v |                 x             |
C                        |       ---             ----    |
C                        |    -          -               |
C                        |                               |
C                  (0,0)  -o-----------------------------
C
C                                Front of the mill
C
C                           PLAN VIEW as mounted on mill.
C
C
          parameter (nslit_max=100, nmax=300, xoffset=0.0) 
          real x(nslit_max,nmax),y(nslit_max,nmax),xx(nmax),yy(nmax)
          character*40 filename
          character*60 string
          logical more
          integer statio,eof,number(nslit_max)
          eof=-1
          more=.true.
C
          write(*,*)' Input name of slit-mask mill file'
          read(*,100)filename
  100     format(a40)
  101     format(a60)
          open(unit=10,file=filename)
C
C    Slits are separated by "newrow" 
C
          nslit=1
          npos=1
          do while (more)
               read(10,101,iostat=statio)string
           write(*,101)string
               if (statio.eq.eof) then
                     more=.false.
               else
                     if (string(1:1) .eq. '(') then
                         go to 200
                     end if
                     if (string(1:6) .eq. 'newrow') then
                         number(nslit)=npos-1
                         nslit=nslit+1
                         npos=1
                     else
                         read(string,*)x(nslit,npos),y(nslit,npos)
                         npos=npos+1
                     end if
               end if
  200          continue
          end do
          nslit=nslit-1
          write(*,*)' Read ',nslit,' slits from input file'
          write(*,*)'   '
          write(*,*)
     :    ' Number of slit, X(1),Y(1), number of segments in slit'
          do i=1,nslit
             write(*,102)i,x(i,1),y(i,1),number(i)
          end do
  102     format(3x,i4,3x,2f8.3,5x,i4)
C
C       Make plots, first top of slitmask
C
           yoffset=30.0   
C
C     need this to make center of mask (Y=132.3 mm) be near the centerline 
C     of the paper
C
           short_side_length=7.0
           short_side_beg=0.5
           short_side_end=short_side_beg + short_side_length
           side_long_length=10.0
           side_long_beg=0.5
           side_long_end=side_long_beg+side_long_length
           xnotch1=0.0
           ynotch1=177.80
           xnotch2=25.40
           ynotch2=0.0
           ybottom=0.0+yoffset
           ytop=ybottom+short_side_length*25.40
           xleft=0.0+xoffset
           xright=xleft+side_long_length*25.40
           call pgbegin(0,'/ps',1,1)
           call pgvsize(side_long_beg,side_long_end,
     :          short_side_beg,short_side_end)
           call pgwindow(xleft,xright,ybottom,ytop)
           call pgbox('cnst',0.0,0,'bcnst',0.0,0)
           call pgtext(xleft+10.,ybottom+40.,'Top Half')
           call pgtext(xleft+10.,ybottom+50.,filename)
           call pgsfs(1)
           call pgarro(xleft+20.0,ynotch1,xleft+10.0,ynotch1)
           call pgtext(xleft+30.0,ynotch1,'Notch')
           call pgarro(xnotch2,ynotch2+60.0,xnotch2,ynotch2+35.0)
           call pgtext(xnotch2+10.0,ynotch2+35.0,'Notch')
           do i=1,nslit
               nval=number(i)-1
               do j=1,nval
                  xx(j)=x(i,j)
                  yy(j)=y(i,j)
               end do
               call pgpoly(nval,xx,yy)
            end do
C
C       Plot of bottom of multi-slit mask
C
           xpage2=140.0
           call pgadvance
           xleft=xpage2+xoffset
           xright=xleft+side_long_length*25.40
           call pgvsize(side_long_beg,side_long_end,
     :          short_side_beg,short_side_end)
           call pgwindow(xleft,xright,ybottom,ytop)
           call pgbox('cnst',0.0,0,'bcnst',0.0,0)
           call pgtext(xleft+10.0,ybottom+20.0,'Bottom Half')
           call pgtext(xleft+10.0,ybottom+50.0,filename)
           do i=1,nslit
               nval=number(i)-1
               do j=1,nval
                  xx(j)=x(i,j)
                  yy(j)=y(i,j)
               end do
               call pgpoly(nval,xx,yy)
            end do
            call pgend
            end
