C
C     Assign slits to objects using PGPLOT.
C
C     Modifications:
C
C     Aug-1997 - Only try to automatically assign one of the points  [PLS]
C                in each line segment.
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.            [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.                 [PLS]
C     Oct-1997 - Removed compile-time warning for loop var assign.   [PLS]
C     Oct-1997 - Added (H) as additional help key.                   [PLS]

      LOGICAL FUNCTION ASSIGN_SLITS(OUTPUT, INTERACTIVE, DEVICE,
     :                              HARD_DEVICE, LPRIORITY, THETA,
     :                              SLIT_WID, SLIT_LEN, SLIT_SEP,
     :                              MIRROR_USE, ENG_TEST, VERBOSE,
     :                              INITIAL)

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      CHARACTER OUTPUT*(*),DEVICE*(*),HARD_DEVICE*(*)
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    THETA
      INTEGER*4 LPRIORITY
      LOGICAL   INTERACTIVE,MIRROR_USE,ENG_TEST,VERBOSE,INITIAL

C     Local variables
      CHARACTER TITLE*80,DVAL*32
      INTEGER*4 I,CUR_OBJ
      REAL*4    XMIN,XMAX,YMIN,YMAX
      REAL*4    SLIT_WID_MM,SLIT_LEN_MM,SLIT_SEP_MM
      LOGICAL   PLOT_FLAGS(6),DONE

C     Functions
      LOGICAL   ADD_SLIT,MANUAL_SLITS

      WRITE(*,*)''
      WRITE(*,11)' Assigning slits...'
 11   FORMAT(A)

      DONE = .TRUE.

C     Scale slit paramets.
      SLIT_WID_MM = SLIT_WID * SCALE
      SLIT_LEN_MM = SLIT_LEN * SCALE
      SLIT_SEP_MM = SLIT_SEP * SCALE

C     Returns array INDX such that PRIORS(INDX(I)) is in decreasing order.
      CALL ISORT(NPTS,PRIORS,INDX)

C     Assign slits in order of priority.
      NSLITS = 0
      I = 1
      DO WHILE (I .LE. NPTS)
         CUR_OBJ = INDX(I)
         IF (ADD_SLIT(CUR_OBJ,THETA,SLIT_LEN_MM,
     :                SLIT_SEP_MM,MIRROR_USE,ENG_TEST)) THEN
            IF (VERBOSE) THEN
               WRITE(*,51)CUR_OBJ,ONAMES(CUR_OBJ)
 51            FORMAT(' Adding slit on object #',I4,': ',A)
            END IF
         ELSE
            IF (OBJ_STYPE(CUR_OBJ) .EQ. TILT_SLIT) THEN
               CALL USER_ERR(
     :                   'Unable to assign tilted slit to object: '//
     :                   ONAMES(CUR_OBJ)(1:13),53)
            ELSE IF (OBJ_STYPE(CUR_OBJ) .EQ. ARC_SLIT) THEN
               CALL USER_ERR('Unable to assign arc slit: '//
     :                       ONAMES(CUR_OBJ)(1:13),40)
            ELSE IF (OBJ_STYPE(CUR_OBJ) .EQ. LINE_SLIT) THEN
               CALL USER_ERR('Unable to assign line segment slit: '//
     :                       ONAMES(CUR_OBJ)(1:13),49)
            ELSE IF (OBJ_STYPE(CUR_OBJ) .EQ. BOX_SLIT) THEN
               CALL USER_ERR('Unable to assign box slit to object: '//
     :                       ONAMES(CUR_OBJ)(1:13),50)
            END IF
         END IF

C        If line segment slit, don't try to add rest of segments points.
         IF (OBJ_STYPE(CUR_OBJ) .EQ. LINE_SLIT) THEN
            I = I + LINE_NUM(OBJ_SIND(CUR_OBJ))
         ELSE
            I = I + 1
         END IF
      END DO

C     Compute dimensions of the plot.
      XMIN = X_CENTER - XSIDE / 2.0
      XMAX = X_CENTER + XSIDE / 2.0
      YMIN = Y_CENTER - YSIDE / 2.0
      YMAX = Y_CENTER + YSIDE / 2.0

C     Create title string.
      IF (MIRROR_USE) THEN
         WRITE(TITLE,101) OUTPUT(1:20),ONAMES(GUIDE_OBJ)(1:35)
 101     FORMAT(1X,'Setup: ',A20,' Guide star: ',A35)
      ELSE
         WRITE(TITLE,102) OUTPUT(1:20)
 102     FORMAT(1X,'Setup: ',A20,' (No guide star)')
      END IF

C     Initialize plotting flags.
      PLOT_FLAGS(1) = .TRUE.    ! plot slitmask frame
      PLOT_FLAGS(2) = MIRROR_USE ! plot pickoff mirror?
      PLOT_FLAGS(3) = .TRUE.    ! plot arrows
      PLOT_FLAGS(4) = .TRUE.    ! plot objects
      PLOT_FLAGS(5) = .TRUE.    ! plot slits
      PLOT_FLAGS(6) = .TRUE.    ! flip X axis

C     Draw slitmask and edit slits interactively.
      IF (INTERACTIVE) THEN
         IF (INITIAL) THEN
            WRITE(*,*)''
            WRITE(*,121)' Starting interactive mode...'
 121        FORMAT(A)

C           Setup plotting devices.
            DEVICE = DEF_DEVICE
            WRITE(*,*)''
            WRITE(*,151) DEVICE(1:6)
 151        FORMAT(' Name of the PGPLOT soft device [',A6,']  ',$)
            READ(*,FMT='(A)') DVAL
            IF (DVAL .NE. ' ') DEVICE = DVAL

C           Initialize drawing device.
            IF (DEVICE .NE. NULL_DEVICE) THEN
               CALL PGBEG(0,DEVICE,1,1)
               CALL PGASK(.FALSE.)
            END IF
         ELSE
            CALL PGERAS()
         END IF

C        Manipulate slits interactively.
         IF (DEVICE .NE. NULL_DEVICE) THEN
            WRITE(*,*)''
            WRITE(*,211)' Interactive mode.  Press H or ? for help...'
 211        FORMAT(A)
            DONE = MANUAL_SLITS(XMIN,XMAX,YMIN,YMAX,TITLE,
     :                          LPRIORITY,THETA,SLIT_WID_MM,SLIT_LEN_MM,
     :                          SLIT_SEP_MM,PLOT_FLAGS,ENG_TEST)

            IF (DONE) THEN
               CALL PGEND
            END IF
         ELSE
            DONE = .TRUE.
         END IF
      END IF

      IF (DONE) THEN
C        Print some summary statistics.
         WRITE(*,*)''
         WRITE(*,310)' ==========================================='
         WRITE(*,310)'                 Slit Summary'
         WRITE(*,310)' -------------------------------------------'
 310     FORMAT(A)
         WRITE(*,311) NSLITS
 311     FORMAT(' Number of slits assigned   =  ',I4)
         WRITE(*,312) NUSE
 312     FORMAT(' Number of objects in field =  ',I4)
         WRITE(*,313) REAL(100 * NSLITS) / REAL(NUSE)
 313     FORMAT(' Percentage of objects hit  = ',F5.1,'%')
         WRITE(*,314)' ==========================================='
 314     FORMAT(A)

C        Lengthen the slits to maximize sky.
         IF (ENG_TEST) THEN
            WRITE(*,*)''
            WRITE(*,401)' *******************************************'
            WRITE(*,401)' Engineering test mode selected.'
            WRITE(*,401)' Not lengthening slits.'
 1          WRITE(*,401)' *******************************************'
 401        FORMAT(A)
         ELSE
            CALL LENGTHEN_SLITS(SLIT_SEP_MM)
         END IF

C        Output final hardcopy plots.
         IF (HARD_DEVICE .NE. NULL_DEVICE) THEN
            CALL DRAW_HARD(XMIN,XMAX,YMIN,YMAX,TITLE,LPRIORITY,
     :                     HARD_DEVICE,THETA,SLIT_WID_MM,MIRROR_USE)
         END IF

C        Clean up and quit.
         WRITE(*,501) NSLITS
 501     FORMAT(' Proceeding with ',I4,' slits...')
      END IF

      ASSIGN_SLITS = DONE
      RETURN
      END
