C
C Calculate the position on the LRIS focal plane detector
C (the CCD) in pixels (xccd, yccd) of an object whose
C position on the sky is (xarc, yarc) in arc - sec.
C
C At PA = 0, +x is East and + Y is North.
C The detector is assumed to be the LRIS detector put into
C service in July 1996. The fit was determined by measuring
C 311 stars an image of M71 taken in September 1996.
C
C J.Cohen / Caltech / June 1997
C
C Aug-13-97 - Changed Y offset following mirror re-alignment.  [JLC]
C

      SUBROUTINE CCD_COORD_PREDICT (XARC, YARC, XCCD, YCCD)
C
C Variables:
C
C input  : xarc, yarc
C output : xccd, yccd
C     

      IMPLICIT NONE

C     Formal parameters
      REAL*8  XARC, YARC, XCCD, YCCD

C     Local variables
      REAL*4  A(10), B(10), YOFFSET
      REAL*8  X2, Y2

      DATA A / 1024.0, -0.47113141E+01, 0.28437639E-02,
     :         0.39969638E-04, -0.61005225E-04, -0.68275855E-05,
     :        -0.50946248E-06,  0.45159715E-07, -0.54639716E-06,
     :         0.56327896E-07 /

      DATA B / 1024.0, 0.41723373E-02, 0.47246534E+01,
     :        -0.57400376E-05, -0.43390770E-05, -0.10129826E-03,
     :         0.25250128E-07,  0.54941795E-06, -0.59603120E-07,
     ;         0.42787942E-06 /

C
C     XCCD = A(1)+A(2)*x+A(3)*y+A(4)*x*x+A(5)*y*y+A(6)*x*y (+3rd order terms)
C     YCCD = B(1)+B(2)*x+B(3)*y+B(4)*x*x+B(5)*y*y+B(6)*x*y (+3rd order terms)
C
      X2 = XARC * XARC
      Y2 = YARC * YARC
      XCCD = A(1) + A(2)*XARC + A(3)*YARC + A(4)*X2 + A(5)*Y2
     :     + A(6)*XARC*YARC + A(7)*XARC*X2 + A(8)*X2*YARC
     :     + A(9)*XARC*Y2 + A(10)*YARC*Y2
      YCCD = B(1) + B(2)*XARC + B(3)*YARC + B(4)*X2 + B(5)*Y2
     :     + B(6)*XARC*YARC + B(7)*XARC*X2 + B(8)*X2*YARC
     :     + B(9)*XARC*Y2 + B(10)*YARC*Y2

C     Add offset in Y (for mis - alignment of mirror relative to gratings).
C     Mirror re-aligned as of August 1997;
C       Y offset changed from 65.0 to 5.0.
C     YOFFSET = 65.0
      YOFFSET = 5.0
      YCCD = YCCD + YOFFSET

      RETURN
      END
