C
C     Output hardcopy plots of slit mask:
C
C         1) slit mask alone
C         2) slit mask + guider holes
C         3) slit mask + guider holes + object/slit labels
C         4) slitmask + matched objects
C
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID             [PLS]

      SUBROUTINE DRAW_HARD(FXMIN,FXMAX,FYMIN,FYMAX,TITLE,LPRIORITY,
     :                     DEVICE,THETA,SLIT_WID,MIRROR_USE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    FXMIN,FXMAX,FYMIN,FYMAX
      REAL*4    SLIT_WID
      REAL*8    THETA
      CHARACTER TITLE*80,DEVICE*(*)
      INTEGER*4 LPRIORITY
      LOGICAL   MIRROR_USE

C     Local variables
      REAL*4       CXR,CYR,OMAG,CLEFT,CRIGHT
      INTEGER*4    CXI,CYI,OBJ,IMAG,IVAR,I
      CHARACTER*4  STRINGX,STRINGY 
      CHARACTER*50 NAME,ASTRING
      CHARACTER*6  MSTRING
      LOGICAL      PLOT_FLAGS(6)

      WRITE(*,*)''
      WRITE(*,11)
 11   FORMAT(' Creating hardcopy...',$)

      CALL PGBEG(0,DEVICE,1,1)

C     Initialize plotting flags.
      PLOT_FLAGS(1) = .TRUE.           ! plot slitmask frame
      PLOT_FLAGS(2) = MIRROR_USE       ! plot pickoff mirror?
      PLOT_FLAGS(3) = .TRUE.           ! plot arrows
      PLOT_FLAGS(4) = .TRUE.           ! plot objects
      PLOT_FLAGS(5) = .TRUE.           ! plot slits
      PLOT_FLAGS(6) = .FALSE.          ! do not flip X axis

C
C     Plot #1: slit mask alone
C
      WRITE(*,101)
 101  FORMAT('1...',$)

      CALL PLOT_MASK(FXMIN,FXMAX,FYMIN,FYMAX,'Mask Layout '//TITLE,
     :               LPRIORITY,THETA,SLIT_WID,PLOT_FLAGS)

      CALL PGPAGE

C
C     Plot #2: slit mask + guider holes
C
      WRITE(*,201)
 201  FORMAT('2...',$)

      PLOT_FLAGS(6) = .TRUE.           ! flip X axis
      CALL PLOT_MASK(FXMIN,FXMAX,FYMIN,FYMAX,'Seen on CCD: '//TITLE,
     :               LPRIORITY,THETA,SLIT_WID,PLOT_FLAGS)

      CALL PGPAGE

C
C     Plot #3: slit mask + guider holes + all object/slit labels
C
      WRITE(*,301)
 301  FORMAT('3...',$)

      CALL PLOT_MASK(FXMIN,FXMAX,FYMIN,FYMAX,'Seen on CCD: '//TITLE,
     :               LPRIORITY,THETA,SLIT_WID,PLOT_FLAGS)

C     Print object names and slit locations.
      DO I=1,NSLITS
         OBJ = SLIT_OBJ(I)

C        Get object position, name, and magnitude
         CXR = OBJ_T_X(OBJ)
         CYR = OBJ_T_Y(OBJ)
         CXI = INT(CXR)
         CYI = INT(CYR)
         NAME = ONAMES(OBJ)
         OMAG = MAGS(OBJ)
         IMAG = INT(OMAG*100.)

C        Print slit position in left margin
         IVAR=3
         CALL PGNUMB(CXI,0,1,STRINGX,IVAR)
         IVAR=4
         CALL PGNUMB(CYI,0,1,STRINGY,IVAR)
         ASTRING = STRINGX(1:3)//','//STRINGY(1:4)
         CLEFT = FXMAX + 0.16 * (FXMAX-FXMIN)
         CALL PGTEXT(CLEFT,CYR,ASTRING(1:8))

C        Print object name and magnitude in right margin
         IVAR=5
         CALL PGNUMB(IMAG,-2,1,MSTRING,IVAR)
         ASTRING = NAME(1:10)//'  '//MSTRING(1:5)
         CRIGHT = FXMIN - 0.03 * (FXMAX-FXMIN)
         CALL PGTEXT(CRIGHT,CYR,ASTRING(1:17))
      END DO

      CALL PGPAGE

C
C     Plot #4: slitmask + matched objects
C
      WRITE(*,401)
 401  FORMAT('4...')

      PLOT_FLAGS(4) = .FALSE.          ! do not plot objects
      CALL PLOT_MASK(FXMIN,FXMAX,FYMIN,FYMAX,'Seen on CCD: '//TITLE,
     :               LPRIORITY,THETA,SLIT_WID,PLOT_FLAGS)

C     Plot matched objects with names.
      DO I=1,NSLITS
         OBJ = SLIT_OBJ(I)

C        Plot object, except arc and line segment slits.
         CALL PLOT_OBJ(OBJ,LPRIORITY,6,3,20,21)

C        Print object names and magnitudes.
         NAME = ONAMES(OBJ)
         OMAG = MAGS(OBJ)
         IMAG = INT(OMAG*100.)
         IVAR=5
         CALL PGNUMB(IMAG,-2,1,MSTRING,IVAR)
         ASTRING = NAME(1:10)//'  '//MSTRING(1:5)
         CALL PGTEXT(OBJ_T_X(OBJ)-5.0,OBJ_T_Y(OBJ),ASTRING)
      END DO

      CALL PGEND

      RETURN
      END
