/* File: env.c */
/* Description: Subroutines to obtain environment information. */
 
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <pwd.h>
#include <netdb.h>


#define UNKNOWN_DOMAIN		"???.edu"


/* Retrieve user's login name and full name. */
void get_user_ (login, llen, name, nlen)
char *login, *name;
int *llen, *nlen;
{
  uid_t uid;
  struct passwd *pwentry;
  int i;

  uid = getuid ();
  pwentry = getpwuid (uid);
  (void) strncpy (login, pwentry->pw_name, *llen);
  (void) strncpy (name, pwentry->pw_gecos, *nlen);

  /* Get rid of '\0's for FORTRAN. */
  for (i = strlen(pwentry->pw_name); i < *llen; i++)
    login[i] = ' ';
  for (i = strlen(pwentry->pw_gecos); i < *nlen; i++)
    name[i] = ' ';

  return;
}


/* Retrieve fully-qualified host name. */
void get_host_ (host, hlen)
char *host;
int *hlen;
{
  char hname[MAXHOSTNAMELEN+2];
  struct hostent *hostentry;
  int i;

  (void) gethostname (hname, MAXHOSTNAMELEN+1);
  (void) strcat (hname, ".");
  hostentry = gethostbyname (hname);

  /* Hostname found. */
  if (hostentry != NULL) {
    (void) strncpy (host, hostentry->h_name, *hlen);
    host[(*hlen)-1] = '\0';
  }

  /* Hostname not found. */
  else {
    (void) strncpy (host, hname, *hlen);
    host[(*hlen)-1] = '\0';
    if ((strlen(host) + strlen(UNKNOWN_DOMAIN)) < *hlen)
      (void) strcat (host, UNKNOWN_DOMAIN);
  }

  /* Get rid of '\0's for FORTRAN. */
  for (i = strlen(host); i < *hlen; i++)
    host[i] = ' ';

  return;
}
