C
C     Load object data from an input file.
C
C     Oct-1997 - Changed EPOCHS_CENTER to EQUINOX_CENTER.          [PLS]
C
      SUBROUTINE GET_DATA(INPUT)

C     Fields to be read include:
C     
C     NPTS           Number of objects 
C     OBJ            Object name
C     RA,DEC         Object position [radians]
C     DRA,DDEC       Object proper motion [arcsec/year]
C     EPOCH          Epoch of input coordinates
C     EQUINOX        Equinox of input coordinates
C     PRIOR          Priority of input object
C     MAG            Object magnitude
C
C     DRA0,DDEC0     Field center [radians]

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      CHARACTER INPUT*40

C     Local variables: object info
      CHARACTER OBJ*40
      INTEGER*4 PRIOR
      REAL*8    RA,DEC
      REAL*4    DRA,DDEC,EPOCH,EQUINOX,MAG

C     Local variables: misc
      CHARACTER OLINE*255
      INTEGER   STATUS(13),IOS,OBJLEN,I

C     Functions
      INTEGER   STAT,GET_OBJ

C     Open input object file
      IOS=STAT(INPUT,STATUS)
      IF (IOS .NE. 0) THEN
         WRITE(*,10) INPUT
 10      FORMAT(' Unable to open input object file: ',A)
         STOP 'Aborting...'
      END IF
      OPEN(12,FILE=INPUT,IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         WRITE(*,11) INPUT
 11      FORMAT(' Unable to open input object file: ',A)
         STOP 'Aborting...'
      END IF

C     Read object data from file.
      WRITE(*,*)''
      WRITE(*,51)' Reading object data file...'
 51   FORMAT(A)
      I=0
      DO WHILE (.TRUE.)
 100     READ(12,FMT='(A)',END=500,ERR=900) OLINE
         OBJLEN = GET_OBJ(OLINE,OBJ,RA,DEC,DRA,DDEC,
     :                    EPOCH,EQUINOX,PRIOR,MAG)
         IF (OBJLEN .EQ. 0) THEN
            GOTO 100                           ! blank line
         ELSE IF (OBJLEN .GT. 0) THEN          ! line read ok
            IF (PRIOR .EQ. 9999) THEN          ! center of field entry
               DRA0 = RA
               DDEC0 = DEC
               EQUINOX_CENTER = EQUINOX
            ELSE                               ! standard object entry
               IF (I .LT. MAXOBJ) THEN
                  I = I + 1
                  ONAMES(I)    = OBJ(1:OBJLEN)
                  RAS(I)       = RA
                  DECS(I)      = DEC
                  DRAS(I)      = DRA
                  DDECS(I)     = DDEC
                  EPOCHS(I)    = EPOCH
                  EQUINOXS(I)  = EQUINOX
                  PRIORS(I)    = PRIOR
                  MAGS(I)      = MAG
                  LINE(I)      = OLINE
                  OBJ_STYPE(I) = STD_SLIT
               ELSE
                  WRITE(*,231)
     :                 ' WARNING: maximum number of objects exceeded.'
 231              FORMAT(A)
                  GOTO 500
               END IF
            END IF
         END IF
      END DO

 500  NPTS = I
      NUSE = NPTS

C     Clean up and quit
      WRITE(*,551) NUSE
 551  FORMAT(' Proceeding with ',I4,' objects...')
      CLOSE(12)
      RETURN

C     Error encountered...
 900  WRITE(*,911) INPUT
 911  FORMAT(' Error reading input object file: ',A)
      CLOSE(12)
      STOP 'Aborting...'
      END
