C
C     Match special slits with objects.
C     Create final ordered list of all slits.
C
C     Modifications:
C
C     Aug-1997 - Increment priorities of specials, so no two slits  [PLS]
C                have exactly the same priority.

      SUBROUTINE MATCH_SLITS(MIRROR_USE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      LOGICAL   MIRROR_USE

C     Local variables: misc
      INTEGER   ICUR,NITER,NSEG,ISEG,I,J
      LOGICAL   FOUND

C     Build sorted object list: boxes, tilted slits, arc slits,
C     line segment slits, guide stars, and rest of objects.
C     NOTE: We don't really *need* to sort the object list.  This
C           process is done mainly for historical reasons, although
C           it does make the object/slit matching slightly faster,
C           since objects that have already been matched with special
C           slits are not examined in future searches.
      WRITE(*,*)''
      WRITE(*,11)' Sorting special slits and matching objects...'
 11   FORMAT(A)
      ICUR = 1

C     Match boxes and insert at beginning of list
      NITER = NBOX
      DO I=1,NITER
         FOUND = .FALSE.
         J = ICUR
         DO WHILE ((J .LE. NPTS) .AND. (.NOT. FOUND))
            IF (BOX_OBJ_NAME(I) .EQ. ONAMES(J)) THEN
C              Place object at current position
               CALL OBJSWAP (ICUR, J)

C              Set object slit type to box
               PRIORS(ICUR) = BOX_PRIOR + (NBOX - I)
               OBJ_STYPE(ICUR) = BOX_SLIT
               OBJ_SIND(ICUR) = I
               BOX_OBJ(I) = ICUR

               ICUR = ICUR + 1
               FOUND = .TRUE.
            END IF
            J = J + 1
         END DO
         IF (.NOT. FOUND) THEN
            CALL USER_ERR('Box not found in object list: '//
     :                    BOX_OBJ_NAME(I),70)
            BOX_OBJ(I) = 0
            NBOX = NBOX - 1
         END IF
      END DO

C     Match tilted slits and insert in list
      NITER = NTILT
      DO I=1,NITER
         FOUND = .FALSE.
         J = ICUR
         DO WHILE ((J .LE. NPTS) .AND. (.NOT. FOUND))
            IF (TILT_OBJ_NAME(I) .EQ. ONAMES(J)) THEN
C              Place object at current position
               CALL OBJSWAP(ICUR, J)

C              Set object slit type to tilt
               PRIORS(ICUR) = TILT_PRIOR + (NTILT - I)
               OBJ_STYPE(ICUR) = TILT_SLIT
               OBJ_SIND(ICUR) = I
               TILT_OBJ(I) = ICUR

               ICUR = ICUR + 1
               FOUND = .TRUE.
            END IF
            J = J + 1
         END DO
         IF (.NOT. FOUND) THEN
            CALL USER_ERR('Tilted slit not found in object list: '//
     :                    TILT_OBJ_NAME(I),78)
            TILT_OBJ(I) = 0
            NTILT = NTILT - 1
         END IF
      END DO

C     Insert arc slits in list
      DO I=1,NARCS
         IF (NPTS .LT. MAXOBJ) THEN
            NPTS = NPTS + 1

C           Place object at current position
            CALL OBJSWAP(ICUR,NPTS)
            ONAMES(ICUR) = ARC_NAME(I)
            RAS(ICUR) = ARC_RA(I)
            DECS(ICUR) = ARC_DEC(I)
            EPOCHS(ICUR) = ARC_EPOCH(I)
            EQUINOXS(ICUR) = ARC_EPOCH(I)
            PRIORS(ICUR) = ARC_PRIOR + (NARCS - I)

C           Set object slit type to arc
            OBJ_STYPE(ICUR) = ARC_SLIT
            OBJ_SIND(ICUR) = I
            ARC_OBJ(I) = ICUR

            ICUR = ICUR + 1
         ELSE
            CALL USER_ERR('Maximum number of objects exceeded: '//
     :                    ARC_NAME(I),76)
            GOTO 500
         END IF
      END DO

C     Insert line segment slits in list
C     First, count number of line segments
      ISEG = 0
      I = 1
      DO WHILE (I .LE. NLINES)
         ISEG = ISEG + 1
         I = I + LINE_NUM(I)
      END DO

      NSEG = 0
      DO I=1,NLINES
C        Maintain position within each segment
         IF (NSEG .EQ. 0) THEN
            NSEG = LINE_NUM(I)
            ISEG = ISEG - 1
         END IF

         IF (NPTS .LT. MAXOBJ) THEN
            NPTS = NPTS + 1

C           Place object at current position
            CALL OBJSWAP(ICUR,NPTS)
            ONAMES(ICUR) = LINE_NAME(I)
            RAS(ICUR) = LINE_RA(I)
            DECS(ICUR) = LINE_DEC(I)
            EPOCHS(ICUR) = LINE_EPOCH(I)
            EQUINOXS(ICUR) = LINE_EPOCH(I)
            PRIORS(ICUR) = LINE_PRIOR + ISEG

C           Set object slit type to line segment
            OBJ_STYPE(ICUR) = LINE_SLIT
            OBJ_SIND(ICUR) = I
            LINE_OBJ(I) = ICUR

            ICUR = ICUR + 1
         ELSE
            CALL USER_ERR('Maximum number of objects exceeded: '//
     :                    LINE_NAME(I),76)
            GOTO 500
         END IF

         NSEG = NSEG - 1
      END DO

C     Match guide stars and insert in list
      IF (MIRROR_USE) THEN
         FOUND = .FALSE.
         J = ICUR
         DO WHILE ((J .LE. NPTS) .AND. (.NOT. FOUND))
            IF (GUIDE_STAR_NAME .EQ. ONAMES(J)) THEN
               CALL OBJSWAP(ICUR, J)
               GUIDE_OBJ = ICUR
               ICUR = ICUR + 1
               FOUND = .TRUE.
            END IF
            J = J + 1
         END DO
         IF (.NOT. FOUND) THEN
            CALL USER_ERR('Guide star not found in object list: '//
     :                    GUIDE_STAR_NAME,77)
         END IF
      END IF

      NSPECIAL = NBOX + NTILT + NARCS + NLINES
      IF (MIRROR_USE) THEN
         NSPECIAL = NSPECIAL + 1
      END IF
      NUSE = NPTS

C     Clean up and quit
 500  WRITE(*,551) NUSE
 551  FORMAT(' Proceeding with ',I4,' positions...')
      RETURN

      END


C
C     Swap object array elements I and J.
C
      SUBROUTINE OBJSWAP(I,J)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      INTEGER   I,J

C     Local variables
      REAL*8    TD
      REAL*4    TR
      INTEGER   TI
      CHARACTER TLINE*254,TNAME*40

      IF (I .EQ. J) THEN
         RETURN
      END IF

      TD = RAS(I)
      RAS(I) = RAS(J)
      RAS(J) = TD

      TD = DECS(I)
      DECS(I) = DECS(J)
      DECS(J) = TD

      TD = RAPREC(I)
      RAPREC(I) = RAPREC(J)
      RAPREC(J) = TD

      TD = DECPREC(I)
      DECPREC(I) = DECPREC(J)
      DECPREC(J) = TD

      TR = DRAS(I)
      DRAS(I) = DRAS(J)
      DRAS(J) = TR

      TR = DDECS(I)
      DDECS(I) = DDECS(J)
      DDECS(J) = TR

      TR = EPOCHS(I)
      EPOCHS(I) = EPOCHS(J)
      EPOCHS(J) = TR

      TR = EQUINOXS(I)
      EQUINOXS(I) = EQUINOXS(J)
      EQUINOXS(J) = TR

      TR = MAGS(I)
      MAGS(I) = MAGS(J)
      MAGS(J) = TR

      TI = PRIORS(I)
      PRIORS(I) = PRIORS(J)
      PRIORS(J) = TI

      TI = OBJ_STYPE(I)
      OBJ_STYPE(I) = OBJ_STYPE(J)
      OBJ_STYPE(J) = TI

      TI = OBJ_SIND(I)
      OBJ_SIND(I) = OBJ_SIND(J)
      OBJ_SIND(J) = TI

      TLINE = LINE(I)
      LINE(I) = LINE(J)
      LINE(J) = TLINE

      TNAME = ONAMES(I)
      ONAMES(I) = ONAMES(J)
      ONAMES(J) = TNAME

      END
