C
C     Load autoslit parameters from an input file.
C
C     Modifications:
C
C     Aug-1997 - Added checks for MAXTILTS, MAXBOXES, etc.          [PLS]
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.           [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.                [PLS]
C     Sep-1997 - Add <= 0 check to SLIT_* inputs.                   [PLS]
C     Oct-1997 - Add ability to specify input file on command line. [PLS]
C     Oct-1997 - Correct usage of DEF_INPUT and DEF_OBJ.            [PLS]

      SUBROUTINE LOAD_PARAMS(INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                       RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                       SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE, LAT,
     :                       POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                       INTERACTIVE, VERBOSE, HARD_DEVICE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

      INCLUDE 'math.inc'

C     Formal parameters
      CHARACTER INPUT*40,OUTPUT*40,OUT2*40,OUT3*40,HARD_DEVICE*32
      INTEGER*4 PRIORITY,RANK_TOP,RANK_BOT
      REAL*4    NEW_EPOCH
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    TEMP,PRES,WAVE,LAT,POS_ANGLE,H
      LOGICAL   MIRROR_USE,ENG_TEST,INTERACTIVE,VERBOSE

C     Local variables: date
      INTEGER*4 DATE(3)

C     Local variables: misc
      CHARACTER PFILE*40,KEY*40,CVAL*64,PLINE*255,NAME*40,CH*1
      INTEGER*4 STATUS(13),IOS,IVAL,KEYLEN,CLEN,VLEN
      INTEGER*4 NAMELEN,I,N,NREAD,NS
      REAL*4    RVAL,PA1,PA2,TPA
      REAL*8    DVAL
      LOGICAL   SLIT_WID_SET,SLIT_LEN_SET,SLIT_SEP_SET,INITIAL

C     Functions
      INTEGER   IARGC,STAT,GET_TILT,GET_ARC,GET_LINE,GET_BOX

C     Set parameters to default values
      INPUT = DEF_OBJ
      OUTPUT = DEF_OUTPUT
      OUT2 = DEF_OUT2
      OUT3 = DEF_OUT3
      PRIORITY = DEF_PRIORITY
      RANK_TOP = DEF_RANK_TOP
      RANK_BOT = DEF_RANK_BOT
      SLIT_WID = DEF_SLIT_WID
      SLIT_LEN = DEF_SLIT_LEN
      SLIT_SEP = DEF_SLIT_SEP
      TEMP = DEF_TEMP
      PRES= DEF_PRES
      WAVE = DEF_WAVE
      LAT = DEF_LAT
      POS_ANGLE = DEF_POS_ANGLE
      H = DEF_H
      MIRROR_USE = DEF_MIRROR_USE
      ENG_TEST = DEF_ENG_TEST
      INTERACTIVE = DEF_INTERACTIVE
      VERBOSE = DEF_VERBOSE
C     Special case: NEW_EPOCH
      CALL IDATE(DATE)
      NEW_EPOCH = FLOAT(DATE(3)) + (30.5*(DATE(2)-1) + DATE(1))/365.25
C     Special slits
      NSLITS = UNDEFINED
      NTILT = 0
      NARCS = 0
      NSEGS = 0
      NBOX = 0
      NLINES = 0
      MTILT = 0
      MARCS = 0
      MSEGS = 0
      MBOX = 0
C     Maximum number of slits, minimum slit length
      MAXSLITS = NORM_SLITS

C     Set flags to avoid engineering mode overrides.
      SLIT_WID_SET = .FALSE.
      SLIT_LEN_SET = .FALSE.
      SLIT_SEP_SET = .FALSE.

C     Get name of autoslit parameter file.
      IF (IARGC() .GT. 0) THEN
C        Get file name from command line.
         CALL GETARG(1,PFILE)
      ELSE IF (STAT(DEF_INPUT,STATUS) .EQ. 0) THEN
C        Try default file name: DEF_INPUT.
         PFILE = DEF_INPUT
      ELSE IF (.NOT. INTERACTIVE) THEN
C        In non-interactive mode, *must* specify file on command line.
C        (This is only executed if DEF_INTERACTIVE is FALSE.)
         WRITE(*,9)'Autoslit parameter file not specified.'
 9       FORMAT(A)
         STOP 'Aborting...'
      ELSE
C        Prompt user for name of autoslit parameter file
         WRITE(*,11)
 11      FORMAT(' Name of autoslit parameter file:  ',$)
         READ(*,FMT='(A)') PFILE
         WRITE(*,*)''
      END IF

C     Open autoslit parameter file
      IOS = STAT(PFILE,STATUS)
      IF (IOS .NE. 0) THEN
          WRITE(*,20) PFILE
 20       FORMAT(' Unable to open input file: ',A)
          STOP 'Aborting...'
      END IF
      OPEN(10,FILE=PFILE,IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
          WRITE(*,21) PFILE
 21       FORMAT(' Unable to open input file: ',A)
          STOP 'Aborting...'
      END IF

C     Read autoslit parameters from file, watching for 'END' string.
      WRITE(*,91)' Reading parameter file...'
 91   FORMAT(A)
      WRITE(*,*)''
 100  DO WHILE (.TRUE.)
         READ(10,FMT='(A)',END=900,ERR=900) PLINE
         CALL GET_KEY(PLINE,KEY,KEYLEN,CVAL,CLEN,IVAL,RVAL,DVAL)

C     Blank line or comment
         IF (KEYLEN .EQ. 0) THEN
            GOTO 100
         END IF

C     Keyword: END
         IF (KEY(1:KEYLEN) .EQ. 'END') THEN
            GOTO 200
         END IF

C     Keyword: FILENAME
         IF (KEY(1:KEYLEN) .EQ. 'FILENAME') THEN
            INPUT = CVAL(1:CLEN)

C     Keyword: FILEOUT
         ELSE IF (KEY(1:KEYLEN) .EQ. 'FILEOUT') THEN
            OUTPUT = CVAL(1:CLEN)
            VLEN = INDEX(CVAL,'.')
            IF (VLEN .EQ. 0) THEN
               VLEN = CLEN
            END IF
            OUT2 = CVAL(1:VLEN)//SUFFIX2
            OUT3 = CVAL(1:VLEN)//SUFFIX3
            HARD_DEVICE = CVAL(1:VLEN)//SUFFIX4//DEF_HDEVICE

C     Keyword: PRIORITY
         ELSE IF (KEY(1:KEYLEN) .EQ. 'PRIORITY') THEN
            PRIORITY = IVAL

C     Keyword: RANK_TOP
         ELSE IF (KEY(1:KEYLEN) .EQ. 'RANK_TOP') THEN
            IF ((IVAL .NE. 1) .AND. (IVAL .NE. 2)) THEN
               GOTO 110
            END IF
            RANK_TOP = IVAL

C     Keyword: RANK_BOT
         ELSE IF (KEY(1:KEYLEN) .EQ. 'RANK_BOT') THEN
            IF ((IVAL .NE. 1) .AND. (IVAL .NE. 2)) THEN
               GOTO 110
            END IF
            RANK_BOT = IVAL

C     Keyword: EPOCH
         ELSE IF (KEY(1:KEYLEN) .EQ. 'EPOCH') THEN
            NEW_EPOCH = RVAL

C     Keyword: SLIT_WID (and WSLIT for backwards compatability)
         ELSE IF ((KEY(1:KEYLEN) .EQ. 'SLIT_WID') .OR.
     :            (KEY(1:KEYLEN) .EQ. 'WSLIT')) THEN
            IF (RVAL .LE. 0.0) THEN
               GOTO 110
            END IF
            SLIT_WID = RVAL
            SLIT_WID_SET = .TRUE.

C     Keyword: SLIT_LEN
         ELSE IF (KEY(1:KEYLEN) .EQ. 'SLIT_LEN') THEN
            IF (RVAL .LE. 0.0) THEN
               GOTO 110
            END IF
            SLIT_LEN = RVAL
            SLIT_LEN_SET = .TRUE.

C     Keyword: SLIT_SEP
         ELSE IF (KEY(1:KEYLEN) .EQ. 'SLIT_SEP') THEN
            IF (RVAL .LE. 0.0) THEN
               GOTO 110
            END IF
            SLIT_SEP = RVAL
            SLIT_SEP_SET = .TRUE.

C     Keyword: TEMP
         ELSE IF (KEY(1:KEYLEN) .EQ. 'TEMP') THEN
            IF ((DVAL .LT. -100.0D0) .OR. (DVAL .GT. 100.0D0)) THEN
               GOTO 110
            END IF
            TEMP = DVAL

C     Keyword: PRESS
         ELSE IF (KEY(1:KEYLEN) .EQ. 'PRESS') THEN
            IF ((DVAL .LT. 100.0D0) .OR. (DVAL .GT. 1000.0D0)) THEN
               GOTO 110
            END IF
            PRES = DVAL

C     Keyword: WAVE
         ELSE IF (KEY(1:KEYLEN) .EQ. 'WAVE') THEN
            IF ((DVAL .LT. 1000.0D0) .OR. (DVAL .GT. 9000.0D0)) THEN
               GOTO 110
            END IF
            WAVE = DVAL

C     Keyword: LAT
         ELSE IF (KEY(1:KEYLEN) .EQ. 'LAT') THEN
            IF ((DVAL .LT. -89.0D0) .OR. (DVAL .GT. 89.0D0)) THEN
               GOTO 110
            END IF
            LAT = DVAL

C     Keyword: POS_ANGL
         ELSE IF (KEY(1:KEYLEN) .EQ. 'POS_ANGL') THEN
            IF ((DVAL .LT. -180.0D0) .OR. (DVAL .GT. 180.0D0)) THEN
               GOTO 110
            END IF
            POS_ANGLE = DVAL

C     Keyword: HOUR_ANG
         ELSE IF (KEY(1:KEYLEN) .EQ. 'HOUR_ANG') THEN
            IF ((DVAL .LT. -6.0D0) .OR. (DVAL .GT. 6.0D0)) THEN
               GOTO 110
            END IF
            H = DVAL

C     Keyword: MIRROR
         ELSE IF (KEY(1:KEYLEN) .EQ. 'MIRROR') THEN
            MIRROR_USE = .TRUE.
            GUIDE_STAR_NAME = CVAL(1:CLEN)

C     Keyword: ENG_TEST
         ELSE IF (KEY(1:KEYLEN) .EQ. 'ENG_TEST') THEN
            CH = CVAL(1:1)
            IF ((CH .EQ. 'T') .OR. (CH .EQ. 't') .OR.
     :          (CH .EQ. 'Y') .OR. (CH .EQ. 'y')) THEN
               ENG_TEST = .TRUE.
               MAXSLITS = ENG_SLITS
            ELSE IF ((CH .EQ. 'F') .OR. (CH .EQ. 'f') .OR.
     :               (CH .EQ. 'N') .OR. (CH .EQ. 'n')) THEN
               ENG_TEST = .FALSE.
               MAXSLITS = NORM_SLITS
            ELSE
               GOTO 110
            END IF

C     Keyword: INTERACT
         ELSE IF (KEY(1:KEYLEN) .EQ. 'INTERACT') THEN
            CH = CVAL(1:1)
            IF ((CH .EQ. 'T') .OR. (CH .EQ. 't') .OR.
     :          (CH .EQ. 'Y') .OR. (CH .EQ. 'y')) THEN
               INTERACTIVE = .TRUE.
            ELSE IF ((CH .EQ. 'F') .OR. (CH .EQ. 'f') .OR.
     :               (CH .EQ. 'N') .OR. (CH .EQ. 'n')) THEN
               INTERACTIVE = .FALSE.
            ELSE
               GOTO 110
            END IF

C     Keyword: VERBOSE
         ELSE IF (KEY(1:KEYLEN) .EQ. 'VERBOSE') THEN
            CH = CVAL(1:1)
            IF ((CH .EQ. 'T') .OR. (CH .EQ. 't') .OR.
     :          (CH .EQ. 'Y') .OR. (CH .EQ. 'y')) THEN
               VERBOSE = .TRUE.
            ELSE IF ((CH .EQ. 'F') .OR. (CH .EQ. 'f') .OR.
     :               (CH .EQ. 'N') .OR. (CH .EQ. 'n')) THEN
               VERBOSE = .FALSE.
            ELSE
               GOTO 110
            END IF

C     Unknown keyword
         ELSE
            CALL USER_ERR('Unknown keyword: '//KEY(1:KEYLEN),
     :                    18+KEYLEN)
         END IF
      END DO

 110  CALL USER_ERR('Parameter outside valid range: '//KEY(1:KEYLEN),
     :                    32+KEYLEN)

      GOTO 100

C     Final parameter settings
 200  IF (ENG_TEST) THEN
         IF (.NOT. SLIT_WID_SET)
     :      SLIT_WID = ENG_SLIT_WID
         IF (.NOT. SLIT_LEN_SET)
     :      SLIT_LEN = ENG_SLIT_LEN
         IF (.NOT. SLIT_SEP_SET)
     :      SLIT_SEP = ENG_SLIT_SEP
         WRITE(*,*)''
         WRITE(*,203)' *******************************************'
         WRITE(*,203)' Engineering test mode selected.'
         WRITE(*,202) SLIT_WID,SLIT_LEN
 202     FORMAT(' Using ',F5.2,'x',F5.2,' arc-sec slits.')
         WRITE(*,203)' *******************************************'
 203     FORMAT(A)
      END IF

C     Read special slit declarations from file, watching for 'END' string.
 204  DO WHILE (.TRUE.)
         READ(10,FMT='(A)',END=900,ERR=900) PLINE
         CALL GET_KEY(PLINE,KEY,KEYLEN,CVAL,CLEN,IVAL,RVAL,DVAL)

C     Blank line or comment
         IF (KEYLEN .EQ. 0) THEN
            GOTO 204
         END IF

C     Keyword: END
         IF (KEY(1:KEYLEN) .EQ. 'END') THEN
            GOTO 500

C     If engineering mode, then no special slits allowed.
         ELSE IF (ENG_TEST) THEN
            WRITE(*,*)''
            WRITE(*,205)' *******************************************'
            WRITE(*,205)' Engineering test mode selected.'
            WRITE(*,205)' No special slits allowed.'
            WRITE(*,205)' *******************************************'
 205        FORMAT(A)
            GOTO 500
         END IF

C     Tilted slits
         IF (KEY(1:KEYLEN) .EQ. 'TILTS') THEN
            N = IVAL
            NREAD = 0
            DO WHILE (NREAD .LT. N)
 210           READ(10,FMT='(A)',END=900,ERR=900) PLINE
               NAMELEN = GET_TILT(PLINE, NAME, TILT(NTILT+1),
     :                            TILT_MIN_UP(NTILT+1),
     :                            TILT_MIN_DOWN(NTILT+1))

               IF (NAMELEN .EQ. 0) THEN
                  GOTO 210      ! blank line or comment
               END IF

               NREAD = NREAD + 1
               IF (NREAD .GT. MAXTILTS) THEN
                  WRITE(*,212) NREAD
 212              FORMAT
     :              (' MAXTILTS exceeded; ignoring tilted slit #',I2)

               ELSE IF (NAMELEN .LT. 0) THEN
                  NREAD = NREAD + 1
                  WRITE(*,214) NREAD
 214              FORMAT(' Error reading tilted slit #',I2)

               ELSE
                  NTILT = NTILT + 1

C                 Scale values from arcsec to mm on mask.
                  TILT_OBJ_NAME(NTILT) = NAME(1:NAMELEN)
                  TILT_MIN_UP(NTILT) = TILT_MIN_UP(NTILT) * SCALE
                  TILT_MIN_DOWN(NTILT) = TILT_MIN_DOWN(NTILT) * SCALE

C                 Ensure position angle limits -180 to +180.
                  PA1 = TILT(NTILT)
                  DO WHILE (PA1 .LT. -180.0)
                     PA1 = PA1 + 360.0
                  END DO
                  DO WHILE (PA1 .GT. +180.0)
                     PA1 = PA1 - 360.0
                  END DO
                  TILT(NTILT) = PA1 * RPD
               END IF
            END DO

C     Arc slits
         ELSE IF (KEY(1:KEYLEN) .EQ. 'ARCS') THEN
            N = IVAL
            NREAD = 0
            DO WHILE (NREAD .LT. N)
 220           READ(10,FMT='(A)',END=900,ERR=900) PLINE
               NAMELEN = GET_ARC(PLINE, NAME, ARC_RA(NARCS+1),
     :                           ARC_DEC(NARCS+1), ARC_EPOCH(NARCS+1),
     :                           ARC_RADIUS(NARCS+1),
     :                           ARC_MIN_DOWN(NARCS+1),
     :                           ARC_MIN_UP(NARCS+1))

               IF (NAMELEN .EQ. 0) THEN
                  GOTO 220      ! blank line or comment
               END IF

               NREAD = NREAD + 1
               IF (NREAD .GT. MAXARCS) THEN
                  WRITE(*,222) NREAD
 222              FORMAT(' MAXARCS exceeded; ignoring arc slit #',I2)

               ELSE IF (NAMELEN .LT. 0) THEN
                  NREAD = NREAD + 1
                  WRITE(*,224) NREAD
 224              FORMAT(' Error reading arc slit #',I2)

               ELSE
                  NARCS = NARCS + 1

C                 Scale values from arcsec to mm on mask.
                  ARC_NAME(NARCS) = NAME(1:NAMELEN)
                  ARC_RADIUS(NARCS) = ARC_RADIUS(NARCS) * SCALE

C                 Arrange position angles.
                  PA1 = ARC_MIN_DOWN(NARCS)
                  PA2 = ARC_MIN_UP(NARCS)

C                 Ensure position angle limits -180 to +180.
                  DO WHILE (PA1 .LT. -180.0)
                     PA1 = PA1 + 360.0
                  END DO
                  DO WHILE (PA2 .LT. -180.0)
                     PA2 = PA2 + 360.0
                  END DO
                  DO WHILE (PA1 .GT. +180.0)
                     PA1 = PA1 - 360.0
                  END DO
                  DO WHILE (PA2 .GT. +180.0)
                     PA2 = PA2 - 360.0
                  END DO

C                 Order position angle limits.
                  IF (PA1 .GT. PA2) THEN
                     TPA = PA1
                     PA1 = PA2
                     PA2 = TPA
                  END IF
                  IF (PA1 .LE. 0 .AND. PA2 .GE. 0) THEN
                     IF (PA2 - PA1 .GT. 180.0) THEN
                        TPA = PA1 + 360.0
                        PA1 = PA2
                        PA2 = TPA
                     END IF
                  END IF

                  ARC_MIN_DOWN(NARCS) = PA1 * RPD
                  ARC_MIN_UP(NARCS) = PA2 * RPD
               END IF
            END DO

C     Line segment slits
         ELSE IF (KEY(1:KEYLEN) .EQ. 'SEGMENTS') THEN
            INITIAL = .TRUE.
            N = IVAL
            NREAD = 0
            DO WHILE (NREAD .LT. N)
 230           READ(10,FMT='(A)',END=900,ERR=900) PLINE
               NAMELEN = GET_LINE(PLINE, NAME, LINE_RA(NLINES+1),
     :                            LINE_DEC(NLINES+1),
     :                            LINE_EPOCH(NLINES+1))

               IF (NAMELEN .EQ. 0) THEN
                  GOTO 230      ! blank line or comment
               END IF

               NREAD = NREAD + 1
               IF (NREAD .GT. MAXLINES) THEN
                  WRITE(*,232) NREAD
 232              FORMAT
     :          (' MAXLINES exceeded; ignoring line segment slit #',I2)

               ELSE IF (NAMELEN .LT. 0) THEN
                  WRITE(*,234) NREAD
 234              FORMAT(' Error reading line segment slit #',I2)

               ELSE
                  NLINES = NLINES + 1

                  LINE_NAME(NLINES) = NAME(1:NAMELEN)
C                 Use slit names to assign segments to slit objects
                  IF (INITIAL) THEN
                     NS = 1
                     NSEGS = NSEGS + 1
                     LINE_INIT(NLINES) = NLINES
                     LINE_NUM(NLINES) = NS
                     INITIAL = .FALSE.
                  ELSE
                     IF (LINE_NAME(NLINES).EQ.LINE_NAME(NLINES-1)) THEN
                        LINE_INIT(NLINES) = LINE_INIT(NLINES-1)
                        NS = NS + 1
                        DO I=0,NS-1
                           LINE_NUM(NLINES-I) = NS
                        END DO
                     ELSE
                        NS = 1
                        NSEGS = NSEGS + 1
                        LINE_INIT(NLINES) = NLINES
                        LINE_NUM(NLINES) = NS
                     END IF
                  END IF
               END IF
            END DO

C     Box slits
         ELSE IF (KEY(1:KEYLEN) .EQ. 'BOXES') THEN
            N = IVAL
            NREAD = 0
            DO WHILE (NREAD .LT. N)
 240           READ(10,FMT='(A)',END=900,ERR=900) PLINE
               NAMELEN = GET_BOX(PLINE, NAME, BOX_SIZE(NBOX+1))

               IF (NAMELEN .EQ. 0) THEN
                  GOTO 240      ! blank line or comment
               END IF

               NREAD = NREAD + 1
               IF (NREAD .GT. MAXBOXES) THEN
                  WRITE(*,242) NREAD
 242              FORMAT(' MAXBOXES exceeded; ignoring box slit #',I2)

               ELSE IF (NAMELEN .LT. 0) THEN
                  WRITE(*,244) NREAD
 244              FORMAT(' Error reading box slit #',I2)

               ELSE
                  NBOX = NBOX + 1

C                 Check for defaulted parameters.
C                 Scale values from arcsec to mm on mask.
                  IF (BOX_SIZE(NBOX) .EQ. DEFAULT)
     :                BOX_SIZE(NBOX) = DEF_BOX_SIZE
                  BOX_SIZE(NBOX) = BOX_SIZE(NBOX) * SCALE
                  BOX_OBJ_NAME(NBOX) = NAME(1:NAMELEN)
               END IF
            END DO

C     Unknown slit declaration
         ELSE
            CALL USER_ERR('Unknown keyword: '//KEY(1:KEYLEN),
     :                    18+KEYLEN)
         END IF
      END DO

C     Clean up and quit
 500  CLOSE(10)
      RETURN

C     Error encountered...
 900  WRITE(*,911) PFILE
 911  FORMAT(' Error reading input file: ',A)
      CLOSE(10)
      STOP 'Aborting...'
      END


C
C     Print autoslit parameters for user confirmation.
C
      SUBROUTINE PRINT_PARAMS(INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                       RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                       SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                       LAT, POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                       INTERACTIVE, VERBOSE, LDL, RDL, OLUN)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

      INCLUDE 'math.inc'

C     Formal parameters
      CHARACTER INPUT*40,OUTPUT*40,OUT2*40,OUT3*40,LDL*1,RDL*1
      INTEGER*4 PRIORITY,RANK_TOP,RANK_BOT,OLUN
      REAL*4    NEW_EPOCH
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    TEMP,PRES,WAVE,LAT,POS_ANGLE,H
      LOGICAL   MIRROR_USE,ENG_TEST,INTERACTIVE,VERBOSE

C     Local variables
      CHARACTER REPLY*10
      INTEGER   I

C     Assume output stream OLUN already open.
C     Print autoslit parameters
      WRITE(OLUN,10)
     :      LDL,'                                               ',RDL
      WRITE(OLUN,10)
     :      LDL,'  ===========================================  ',RDL
      WRITE(OLUN,10)
     :      LDL,'              Autoslit Parameters              ',RDL
      WRITE(OLUN,10)
     :      LDL,'  -------------------------------------------  ',RDL

      WRITE(OLUN,10)
     :      LDL,'                                               ',RDL
      WRITE(OLUN,10)
     :      LDL,'  Files:                                       ',RDL
      WRITE(OLUN,10)
     :      LDL,'  -------------------------------------------  ',RDL
 10   FORMAT(A1,A,A1)
      WRITE(OLUN,11) LDL, INPUT, RDL
 11   FORMAT(A1,'  Input object file:              ',A11,'  ',A1)
      WRITE(OLUN,12) LDL, OUTPUT, RDL
 12   FORMAT(A1,'  Output listing file:            ',A11,'  ',A1)
      WRITE(OLUN,13) LDL, OUT2, RDL
 13   FORMAT(A1,'  Output stage coordinate file:   ',A11,'  ',A1)
      WRITE(OLUN,14) LDL, OUT3, RDL
 14   FORMAT(A1,'  Output file for the punch/mill: ',A11,'  ',A1)
      WRITE(OLUN,20)
     :      LDL,'  -------------------------------------------  ',RDL

      WRITE(OLUN,20)
     :      LDL,'                                               ',RDL
      WRITE(OLUN,20)
     ;      LDL,'  Physical parameters:     [default]  [input]  ',RDL
      WRITE(OLUN,20)
     :      LDL,'  -------------------------------------------  ',RDL
 20   FORMAT(A1,A,A1)
      WRITE(OLUN,21) LDL, DEF_TEMP, TEMP, RDL
 21   FORMAT(A1,
     :       '  Temperature [C]:           ',F6.1,'    ',F6.1,'  ',A1)
      WRITE(OLUN,22) LDL, DEF_PRES, PRES, RDL
 22   FORMAT(A1,
     :       '  Air pressure [mm of Hg]:   ',F6.1,'    ',F6.1,'  ',A1)
      WRITE(OLUN,23) LDL, DEF_LAT, LAT, RDL
 23   FORMAT(A1,
     :       '  Latitude:                  ',F6.2,'    ',F6.2,'  ',A1)
      WRITE(OLUN,24) LDL, DEF_WAVE, WAVE, RDL
 24   FORMAT(A1,
     :       '  Wavelength [A]:            ',F6.1,'    ',F6.1,'  ',A1)
      WRITE(OLUN,25) LDL, DEF_POS_ANGLE, POS_ANGLE, RDL
 25   FORMAT(A1,'  Position angle:           ',F7.2,'   ',F7.2,'  ',A1)
      WRITE(OLUN,26) LDL, DEF_H, H, RDL
 26   FORMAT(A1,
     :     '  Hour angle:                 ',F5.2,'     ',F5.2,'  ',A1)
      WRITE(OLUN,27) LDL, NEW_EPOCH, RDL
 27   FORMAT(A1,'  Epoch:                     [today]  ',F7.2,'  ',A1)
      IF (ENG_TEST) THEN
         WRITE(OLUN,28) LDL, ENG_SLIT_WID, SLIT_WID, RDL
      ELSE
         WRITE(OLUN,28) LDL, DEF_SLIT_WID, SLIT_WID, RDL
      END IF
 28   FORMAT(A1,
     :     '  Slit width [arc-sec]:       ',F5.2,'     ',F5.2,'  ',A1)
      IF (ENG_TEST) THEN
         WRITE(OLUN,29) LDL, ENG_SLIT_LEN, SLIT_LEN, RDL
      ELSE
         WRITE(OLUN,29) LDL, DEF_SLIT_LEN, SLIT_LEN, RDL
      END IF
 29   FORMAT(A1,
     :     '  Slit length [arc-sec]:      ',F5.2,'     ',F5.2,'  ',A1)
      IF (ENG_TEST) THEN
         WRITE(OLUN,30) LDL, ENG_SLIT_SEP, SLIT_SEP, RDL
      ELSE
         WRITE(OLUN,30) LDL, DEF_SLIT_SEP, SLIT_SEP, RDL
      END IF
 30   FORMAT(A1,
     :     '  Slit separation [arc-sec]:  ',F5.2,'     ',F5.2,'  ',A1)
      WRITE(OLUN,40)
     :      LDL,'  -------------------------------------------  ',RDL

      WRITE(OLUN,40)
     :      LDL,'                                               ',RDL
      WRITE(OLUN,40)
     :      LDL,'  Slit layout parameters:                      ',RDL
      WRITE(OLUN,40)
     :      LDL,'  -------------------------------------------  ',RDL
      IF (RANK_TOP .GT. 1) THEN
         WRITE(OLUN,40)
     :         LDL,'  Top half of mask:    *double* rank of slits  ',RDL
      ELSE
         WRITE(OLUN,40)
     :         LDL,'  Top half of mask:    *single* rank of slits  ',RDL
      END IF
      IF (RANK_BOT .GT. 1) THEN
         WRITE(OLUN,40)
     :         LDL,'  Bottom half of mask: *double* rank of slits  ',RDL
      ELSE
         WRITE(OLUN,40)
     :         LDL,'  Bottom half of mask: *single* rank of slits  ',RDL
      END IF
 40   FORMAT(A1,A,A1)
      WRITE(OLUN,41) LDL, PRIORITY, RDL
 41   FORMAT(A1,'  Special plot symbols have priority  > ',I5,'  ',A1)
      WRITE(OLUN,42) LDL
 42   FORMAT(A1,'  Slit selection:             ',$)
      IF (INTERACTIVE) THEN
         WRITE(OLUN,43)'    interactive  ',RDL
      ELSE
         WRITE(OLUN,43)'non-interactive  ',RDL
      END IF
 43   FORMAT(A,A1)
      WRITE(OLUN,44) LDL
 44   FORMAT(A1,'  Pickoff mirror in use:                ',$)
      IF (MIRROR_USE) THEN
         WRITE(OLUN,45)' TRUE  ',RDL
      ELSE
         WRITE(OLUN,45)'FALSE  ',RDL
      END IF
 45   FORMAT(A,A1)
      WRITE(OLUN,46) LDL
 46   FORMAT(A1,'  Engineering run:                      ',$)
      IF (ENG_TEST) THEN
         WRITE(OLUN,47)' TRUE  ',RDL
      ELSE
         WRITE(OLUN,47)'FALSE  ',RDL
      END IF
 47   FORMAT(A,A1)
      WRITE(OLUN,60)
     :      LDL,'  -------------------------------------------  ',RDL

      WRITE(OLUN,60)
     :      LDL,'                                               ',RDL
      WRITE(OLUN,60)
     :      LDL,'  Slit inventory:           assigned/declared  ',RDL
      WRITE(OLUN,60)
     :      LDL,'  -------------------------------------------  ',RDL
 60   FORMAT(A1,A,A1)
      IF (NSLITS .NE. UNDEFINED) THEN
         WRITE(OLUN,61) LDL, NSLITS-MSPECIAL, RDL
 61      FORMAT(A1,
     :          '  Number of standard slits:              ',I4,'  ',A1)
      END IF
      WRITE(OLUN,62) LDL, MBOX, NBOX, RDL
 62   FORMAT
     :(A1,'  Number of boxes:                      ',I2,'/',I2,'  ',A1)
      IF (VERBOSE) THEN
         WRITE(OLUN,63)
     :      LDL,'  -------------------------------------------  ',RDL
 63     FORMAT(A1,A,A1)
         IF (NBOX .GT. 0) THEN
            DO I=1,NBOX
               WRITE(OLUN,64) LDL, BOX_OBJ_NAME(I)(1:10),
     :                        BOX_SIZE(I)/SCALE, RDL
 64            FORMAT
     :         (A1,'    ',A10,'  [',F5.2,']                        ',A1)
            END DO
            WRITE(OLUN,65)
     :      LDL,'  -------------------------------------------  ',RDL
            WRITE(OLUN,65)
     :      LDL,'                                               ',RDL
 65         FORMAT(A1,A,A1)
         END IF
      END IF

      WRITE(OLUN,70) LDL, MTILT, NTILT, RDL
 70   FORMAT
     :(A1,'  Number of tilted slits:               ',I2,'/',I2,'  ',A1)
      IF (VERBOSE) THEN
         WRITE(OLUN,71)
     :      LDL,'  -------------------------------------------  ',RDL
 71      FORMAT(A1,A,A1)
         IF (NTILT .GT. 0) THEN
            DO I=1,NTILT
               WRITE(OLUN,72) LDL, TILT_OBJ_NAME(I)(1:10), TILT(I)*DPR,
     :                        TILT_MIN_UP(I)/SCALE,
     :                        TILT_MIN_DOWN(I)/SCALE, RDL
 72            FORMAT(A1,'    ',A10,'  [',F5.1,F8.2,F9.2,']       ',A1)
            END DO
            WRITE(OLUN,73)
     :      LDL,'  -------------------------------------------  ',RDL
            WRITE(OLUN,73)
     :      LDL,'                                               ',RDL
 73         FORMAT(A1,A,A1)
         END IF
      END IF

      WRITE(OLUN,80) LDL, MARCS, NARCS, RDL
 80   FORMAT
     :(A1,'  Number of arc slits:                  ',I2,'/',I2,'  ',A1)
      IF (VERBOSE) THEN
         WRITE(OLUN,81)
     :      LDL,'  -------------------------------------------  ',RDL
 81      FORMAT(A1,A,A1)
         IF (NARCS .GT. 0) THEN
            DO I=1,NARCS
               WRITE(OLUN,82) LDL, ARC_NAME(I)(1:10), ARC_RA(I),
     :                        ARC_DEC(I), ARC_EPOCH(I), RDL
 82            FORMAT(A1,'    ',A10,'  [',F5.2,F8.2,F9.2,']       ',A1)
            END DO
            WRITE(OLUN,83)
     :      LDL,'  -------------------------------------------  ',RDL
            WRITE(OLUN,83)
     :      LDL,'                                               ',RDL
 83         FORMAT(A1,A,A1)
         END IF
      END IF

      WRITE(OLUN,90) LDL, MSEGS, NSEGS, RDL
 90   FORMAT
     :(A1,'  Number of line segment slits:         ',I2,'/',I2,'  ',A1)
      IF (VERBOSE) THEN
         WRITE(OLUN,91)
     :      LDL,'  -------------------------------------------  ',RDL
 91      FORMAT(A1,A,A1)
         IF (NLINES .GT. 0) THEN
            WRITE(OLUN,92) LDL, LINE_NAME(1)(1:10), LINE_RA(1),
     :                     LINE_DEC(1), LINE_EPOCH(1), RDL
 92         FORMAT(A1,'    ',A10,'  [',F5.2,F8.2,F9.2,']       ',A1)
            DO I=2,NLINES
               IF (LINE_NAME(I) .NE. LINE_NAME(I-1)) THEN
                  WRITE(OLUN,93) LDL, LINE_NAME(I)(1:10), LINE_RA(I),
     :                           LINE_DEC(I), LINE_EPOCH(I), RDL
 93               FORMAT(A1,
     :                   '    ',A10,'  [',F5.2,F8.2,F9.2,']       ',A1)
               ELSE
                  WRITE(OLUN,94) LDL, LINE_RA(I), LINE_DEC(I),
     :                           LINE_EPOCH(I), RDL
 94               FORMAT(A1,
     :                '                [',F5.2,F8.2,F9.2,']       ',A1)
               END IF
            END DO
            WRITE(OLUN,95)
     :      LDL,'  -------------------------------------------  ',RDL
            WRITE(OLUN,95)
     :      LDL,'                                               ',RDL
 95         FORMAT(A1,A,A1)
         END IF
      END IF
      WRITE(OLUN,99)
     :      LDL,'  ===========================================  ',RDL
 99   FORMAT(A1,A,A1)

C     Query user to continue.
      IF ((OLUN .EQ. STDOUT) .AND. INTERACTIVE) THEN
         WRITE(OLUN,*)''
         WRITE(OLUN,151)
 151     FORMAT(' Are these values correct?  ',$)
         READ(*,FMT='(A)') REPLY
         IF ((REPLY(1:1) .EQ. 'N') .OR. (REPLY(1:1) .EQ. 'n')) THEN
            STOP 'Exiting...'
         END IF
      END IF

      END


C
C     Report error to user and check for continuation.
C
      SUBROUTINE USER_ERR(MSG, LEN)

      IMPLICIT NONE

C     Local parameters
      CHARACTER MSG*255
      INTEGER*4 LEN
C     Local variables
      CHARACTER REPLY*10

      WRITE(*,101)' *******************************************'
      WRITE(*,101)' '//MSG(1:LEN)
      WRITE(*,101)' *******************************************'
 101  FORMAT(A)
      WRITE(*,121)
 121  FORMAT(' Continue?  ',$)
      READ(*,FMT='(A)') REPLY
      IF ((REPLY(1:1) .EQ. 'N') .OR. (REPLY(1:1) .EQ. 'n')) THEN
         STOP 'Exiting...'
      END IF
      WRITE(*,131)' Ok, ignoring...'
 131  FORMAT(A)
      WRITE(*,*)''

      END
