C
C     Create plot of slitmask.
C     Uses PGPLOT plotting subroutine library.
C
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID             [PLS]

      SUBROUTINE PLOT_MASK(FXMIN,FXMAX,FYMIN,FYMAX,TITLE,LPRIORITY,
     :                     THETA,SLIT_WID,FLAGS)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    FXMIN,FXMAX,FYMIN,FYMAX
      REAL*4    SLIT_WID
      REAL*8    THETA
      CHARACTER TITLE*80
      INTEGER*4 LPRIORITY
      LOGICAL   FLAGS(6)

C     Local variables
      REAL*4    TXMIN,TXMAX
      INTEGER*4 I
      LOGICAL   PLOT_FRAME_FLAG,PLOT_MIRROR_FLAG,PLOT_ARROWS_FLAG
      LOGICAL   PLOT_OBJS_FLAG,PLOT_SLITS_FLAG,FLIP_X_FLAG

C     Set flags.
      PLOT_FRAME_FLAG  = FLAGS(1)
      PLOT_MIRROR_FLAG = FLAGS(2)
      PLOT_ARROWS_FLAG = FLAGS(3)
      PLOT_OBJS_FLAG   = FLAGS(4)
      PLOT_SLITS_FLAG  = FLAGS(5)
      FLIP_X_FLAG      = FLAGS(6)

C     Flip X axis if necessary.
      IF (FLIP_X_FLAG) THEN
         TXMIN = FXMAX
         TXMAX = FXMIN
      ELSE
         TXMIN = FXMIN
         TXMAX = FXMAX
      END IF

C     Draw slit mask frame.
      IF (PLOT_FRAME_FLAG) THEN
         CALL PLOT_FRAME(0.0,1.0,0.06,0.90,
     :                   TXMIN,TXMAX,FYMIN,FYMAX,
     :                   ' ',' ',TITLE)
      END IF

C     Draw central bar in slitmask.
      CALL PGMOVE(TXMIN,BYMIN)
      CALL PGDRAW(TXMAX,BYMIN)
      CALL PGDRAW(TXMAX,BYMAX)
      CALL PGDRAW(TXMIN,BYMAX)
      CALL PGDRAW(TXMIN,BYMIN)

C     Draw pickoff mirror box.
      IF (PLOT_MIRROR_FLAG) THEN
         CALL PGSCI(2)
         CALL PGMOVE(GXMIN,GYMIN)
         CALL PGDRAW(GXMAX,GYMIN)
         CALL PGDRAW(GXMAX,GYMAX)
         CALL PGDRAW(GXMIN,GYMAX)
         CALL PGDRAW(GXMIN,GYMIN)
      END IF

C     Add N,E arrows.
      IF (PLOT_ARROWS_FLAG) THEN
         CALL PLOT_ARROWS(TXMIN,TXMAX,FYMIN,FYMAX,THETA)
      END IF

C     Plot all objects.
      IF (PLOT_OBJS_FLAG) THEN
         DO I=1,NPTS
            CALL PLOT_OBJ(I,LPRIORITY,6,3,20,21)
         END DO
      END IF

C     Draw slits.
      IF (PLOT_SLITS_FLAG) THEN
         DO I = 1,NSLITS
            CALL PLOT_SLIT(I,SLIT_WID,THETA,1)
         END DO
      END IF

      RETURN
      END


C
C     Plot a slit.
C     Uses PGPLOT plotting subroutine library.
C
C     Sep-1997 - Adjusted for lengthening tilted slits.            [PLS]

      SUBROUTINE PLOT_SLIT(SLIT,WIDTH,THETA,COLOR)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      INTEGER*4 SLIT,COLOR
      REAL*4    WIDTH
      REAL*8    THETA

C     Local variables
      INTEGER*4 OBJ,STYPE,SIND,I,J,N
      REAL*4    X1,X2,RAD,HWIDTH
      REAL*4    X(2*MAX_ARC_LEN),Y(2*MAX_ARC_LEN)
      REAL*8    ANGLE,DANGLE

C     Set plot color.
      CALL PGSCI(COLOR)
      HWIDTH = 0.5 * WIDTH

C     Determine corresponding object, slit type, and slit index.
      OBJ = SLIT_OBJ(SLIT)
      STYPE = OBJ_STYPE(OBJ)
      SIND = OBJ_SIND(OBJ)

C     Draw slit, based on slit type.
      IF (STYPE .EQ. STD_SLIT) THEN
C        Standard slit.
         CALL PGMOVE(SLIT_X(SLIT)-HWIDTH,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(SLIT_X(SLIT)-HWIDTH,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(SLIT_X(SLIT)+HWIDTH,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(SLIT_X(SLIT)+HWIDTH,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(SLIT_X(SLIT)-HWIDTH,SLIT_MIN_Y(SLIT))

      ELSE IF (STYPE .EQ. TILT_SLIT) THEN
C        Tilted slit.
C        Note that internal position angle is opposite.
         ANGLE = TILT(SIND) + (THETA * RPD)
         IF (ANGLE .LE. -HALFPI .OR. ANGLE .GE. HALFPI) THEN
            X1 = SLIT_X(SLIT) +
     :           ((SLIT_MIN_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
            X2 = SLIT_X(SLIT) +
     :           ((SLIT_MAX_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
C           This is same calculation, but depends on TILT_MIN_*.
C           X1 = SLIT_X(SLIT) + (TILT_MIN_UP(SIND) * DSIN(ANGLE))
C           X2 = SLIT_X(SLIT) - (TILT_MIN_DOWN(SIND) * DSIN(ANGLE))
         ELSE
            X1 = SLIT_X(SLIT) +
     :           ((SLIT_MIN_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
            X2 = SLIT_X(SLIT) +
     :           ((SLIT_MAX_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
C           This is same calculation, but depends on TILT_MIN_*.
C           X1 = SLIT_X(SLIT) - (TILT_MIN_DOWN(SIND) * DSIN(ANGLE))
C           X2 = SLIT_X(SLIT) + (TILT_MIN_UP(SIND) * DSIN(ANGLE))
         END IF
         CALL PGMOVE(X1-HWIDTH,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(X2-HWIDTH,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(X2+HWIDTH,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(X1+HWIDTH,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(X1-HWIDTH,SLIT_MIN_Y(SLIT))

      ELSE IF (STYPE .EQ. ARC_SLIT) THEN
C        Arc slit.
C        Compute angular extent and number of points.
         RAD = ARC_RADIUS(SIND)
         ANGLE = ARC_MIN_UP(SIND) - ARC_MIN_DOWN(SIND)
         N = INT(ANGLE * RAD) * 2
         DANGLE = ANGLE / (REAL(N-1))
C        Determine point for each angular step.
         ANGLE = ARC_MIN_DOWN(SIND) + (THETA * RPD)
         J = 2*N
         DO I=1,N
            X(I) = OBJ_T_X(OBJ) + (RAD * DSIN(ANGLE)) - HWIDTH
            Y(I) = OBJ_T_Y(OBJ) + (RAD * DCOS(ANGLE))

            X(J) = X(I) + WIDTH
            Y(J) = Y(I)

            ANGLE = ANGLE + DANGLE
            J = J - 1
         END DO
         X(2*N+1) = X(1)
         Y(2*N+1) = Y(1)
         CALL PGLINE(2*N+1,X,Y)

      ELSE IF (STYPE .EQ. LINE_SLIT) THEN
C        Line segment slit.
         N = LINE_NUM(SIND)
         J = 2*N
         DO I=1,N
            X(I) = OBJ_T_X(OBJ+I-1) - HWIDTH
            Y(I) = OBJ_T_Y(OBJ+I-1)

            X(J) = X(I) + WIDTH
            Y(J) = Y(I)
            J = J - 1
         END DO
         X(2*N+1) = X(1)
         Y(2*N+1) = Y(1)
         CALL PGLINE(2*N+1,X,Y)

      ELSE IF (STYPE .EQ. BOX_SLIT) THEN
C        Box slit.
C        NOTE: Box will only be stretched vertically,
C              i.e., aspect ratio not preserved.
         X1 = SLIT_X(SLIT) - (BOX_SIZE(SIND)/2.0)
         X2 = SLIT_X(SLIT) + (BOX_SIZE(SIND)/2.0)
         CALL PGMOVE(X1,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(X2,SLIT_MIN_Y(SLIT))
         CALL PGDRAW(X2,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(X1,SLIT_MAX_Y(SLIT))
         CALL PGDRAW(X1,SLIT_MIN_Y(SLIT))
      END IF

      RETURN
      END


C
C     Plot an object.
C     Uses PGPLOT plotting subroutine library.
C
      SUBROUTINE PLOT_OBJ(OBJ,OPRIORITY,COLORLO,COLORHI,SYMLO,SYMHI)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      INTEGER OBJ,OPRIORITY,COLORLO,COLORHI,SYMLO,SYMHI

C     Set plot point size.
      CALL PGSCH(2.0)

C     Draw point for object.
      IF ((OBJ_STYPE(OBJ) .NE. ARC_SLIT) .AND.
     :    (OBJ_STYPE(OBJ) .NE. LINE_SLIT) .AND.
     :    (.NOT. OBJ_OUT(OBJ))) THEN
         IF (PRIORS(OBJ) .LT. OPRIORITY) THEN
            CALL PGSCI(COLORLO)
            CALL PGPT(1,OBJ_T_X(OBJ),OBJ_T_Y(OBJ),SYMLO)
         ELSE
            CALL PGSCI(COLORHI)
            CALL PGPT(1,OBJ_T_X(OBJ),OBJ_T_Y(OBJ),SYMHI)
         END IF
      END IF
      CALL PGSCH(1.0)

      RETURN
      END

         
C
C     Plot slitmask frame.
C     Uses PGPLOT plotting subroutine library.
C
      SUBROUTINE PLOT_FRAME(VPX1,VPX2,VPY1,VPY2,
     :                      AXX1,AXX2,AXY1,AXY2,
     :                      XLABEL,YLABEL,PTITLE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    VPX1,VPX2,VPY1,VPY2            ! viewport coordinates
      REAL*4    AXX1,AXX2,AXY1,AXY2            ! axes coordinates
      CHARACTER XLABEL*(*),YLABEL*(*),PTITLE*(*)

C     Local variables
      REAL*4    CLENGTH,XCEN,YCEN
      REAL*4    TX1,TY1,TX2,TY2

C     Set plot character size and color.
      CALL PGSCH(1.0)
      CALL PGSCI(1)

C     Draw plot box.
      CALL PGSVP(VPX1,VPX2,VPY1,VPY2)
      CALL PGLAB(XLABEL,YLABEL,PTITLE)
      CALL PGWNAD(AXX1,AXX2,AXY1,AXY2)
      CALL PGBOX('BCNST',0.0,0,'BCNSTV',0.0,0)

C     Draw (two) arrows to mark notches in slitmask frame.
      CALL PGSCH(0.5)                        ! reduce size of arrowhead
      CLENGTH=SQRT((AXX2-AXX1)*(AXX2-AXX1)
     :            +(AXY2-AXY1)*(AXY2-AXY1))
      XCEN = (AXX2 + AXX1) / 2.0
      YCEN = (AXY2 + AXY1) / 2.0

      TX2 = NOTCH1X
      TY2 = NOTCH1Y
      CALL MKXYINBOX (AXX1,AXY1,AXX2,AXY2,TX2,TY2)
      IF (TX2 .GT. XCEN) THEN
         TX1 = TX2 - 0.025 * CLENGTH
      ELSE
         TX1 = TX2 + 0.025 * CLENGTH
      END IF
      TY1 = TY2
      CALL PGARRO(TX1,TY1,TX2,TY2)

      TX2 = NOTCH2X
      TY2 = NOTCH2Y
      CALL MKXYINBOX (AXX1,AXY1,AXX2,AXY2,TX2,TY2)
      TX1 = TX2
      IF (TY2 .GT. YCEN) THEN
         TY1 = TY2 - 0.025 * CLENGTH
      ELSE
         TY1 = TY2 + 0.025 * CLENGTH
      END IF
      CALL PGARRO(TX1,TY1,TX2,TY2)

      RETURN
      END


C
C     Return point within box (x1,y1,x2,y2) that
C     is closest to the specified point (x,y).
C     (Utility subroutine for PLOT_FRAME.)
C
      SUBROUTINE MKXYINBOX(X1,Y1,X2,Y2,X,Y)

      IMPLICIT NONE

C     Formal parameters
      REAL*4 X1,Y1,X2,Y2
      REAL*4 X,Y

      IF (X1 .LT. X2) THEN              ! X1,X2 not necessarily in order
         IF (X .LT. X1) X = X1
         IF (X .GT. X2) X = X2
      ELSE
         IF (X .LT. X2) X = X2
         IF (X .GT. X1) X = X1
      END IF

      IF (Y1 .LT. Y2) THEN              ! Y1,Y2 not necessarily in order
         IF (Y .LT. Y1) Y = Y1
         IF (Y .GT. Y2) Y = Y2
      ELSE
         IF (Y .LT. Y2) Y = Y2
         IF (Y .GT. Y1) Y = Y1
      END IF

      RETURN
      END


C
C     Plot directional arrows.
C     Uses PGPLOT plotting subroutine library.
C
      SUBROUTINE PLOT_ARROWS(AXX1,AXX2,AXY1,AXY2,THETA)

      IMPLICIT NONE

      INCLUDE 'math.inc'

C     Formal parameters
      REAL*4 AXX1,AXX2,AXY1,AXY2            ! axes coordinates
      REAL*8 THETA                          ! position angle

C     Local variables
      REAL*4 VALUEX,VALUEY,CONSTANTX,CONSTANTY
      REAL*4 CONSTANTX2,CONSTANTY2,CONSTANTX3,CONSTANTY3
      REAL*4 CLENGTH
      REAL*8 ANGLE

C     Set plot character size and color.
      CALL PGSCH(1.0)
      CALL PGSCI(1)

C     Add N,E arrows.
      CONSTANTX = AXX1 + 0.1 * (AXX2-AXX1)
      CONSTANTY = AXY1 + 0.9 * (AXY2-AXY1)
      CLENGTH=SQRT((AXX2-AXX1)*(AXX2-AXX1)
     :            +(AXY2-AXY1)*(AXY2-AXY1))

      ANGLE = THETA * RPD
      CONSTANTX2 = CONSTANTX + 0.05 * CLENGTH * DSIN(ANGLE)
      CONSTANTY2 = CONSTANTY + 0.05 * CLENGTH * DCOS(ANGLE)
      CALL PGARRO(CONSTANTX,CONSTANTY,CONSTANTX2,CONSTANTY2)
      IF (CONSTANTX2 .EQ. CONSTANTX) THEN
         VALUEX = CONSTANTX2
      ELSE IF (CONSTANTX2 .LT. CONSTANTX) THEN
         VALUEX = CONSTANTX2 - 0.01 * CLENGTH
      ELSE
         VALUEX = CONSTANTX2 + 0.01 * CLENGTH
      END IF
      IF (CONSTANTY2 .EQ. CONSTANTY) THEN
         VALUEY = CONSTANTY2
      ELSE IF (CONSTANTY2 .LT. CONSTANTY) THEN
         VALUEY = CONSTANTY2 - 0.01 * CLENGTH
      ELSE
         VALUEY = CONSTANTY2 + 0.01 * CLENGTH
      END IF
      CALL PGTEXT(VALUEX,VALUEY,'N')

      ANGLE = (THETA+90.) * RPD
      CONSTANTX3 = CONSTANTX + 0.05 * CLENGTH * DSIN(ANGLE)
      CONSTANTY3 = CONSTANTY + 0.05 * CLENGTH * DCOS(ANGLE)
      CALL PGARRO(CONSTANTX,CONSTANTY,CONSTANTX3,CONSTANTY3)
      IF (CONSTANTX3 .EQ. CONSTANTX) THEN
         VALUEX = CONSTANTX3
      ELSE IF (CONSTANTX3 .LT. CONSTANTX) THEN
         VALUEX = CONSTANTX3 - 0.01 * CLENGTH
      ELSE
         VALUEX = CONSTANTX3 + 0.01 * CLENGTH
      END IF
      IF (CONSTANTY3 .EQ. CONSTANTY) THEN
         VALUEY = CONSTANTY3
      ELSE IF (CONSTANTY3 .LT. CONSTANTY) THEN
         VALUEY = CONSTANTY3 - 0.01 * CLENGTH
      ELSE
         VALUEY = CONSTANTY3 + 0.01 * CLENGTH
      END IF
      CALL PGTEXT(VALUEX,VALUEY,'E')  

      RETURN
      END
