C
C     Precess slit coordinates to specified epoch.
C     Borrowed from CRO's precession program.
C
C     Oct-1997 - Changed to precess EQUINOXS, rather than EPOCHS.  [PLS]
C     Oct-1997 - Changed EPOCHS_CENTER to EQUINOX_CENTER.          [PLS]
C
      SUBROUTINE PRECESS(NEW_EPOCH)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    NEW_EPOCH

C     Local variables
      REAL*4    DRA,DDEC
      INTEGER*4 I

      WRITE(*,*)''
      WRITE(*,11)' Precessing coordinates...'
 11   FORMAT(A)

C     Precess slit coordinates.
      DO I=1,NPTS
         CALL NOR_PRECESS(RAS(I),DECS(I),DRAS(I),DDECS(I),
     :                    EQUINOXS(I),NEW_EPOCH,RAPREC(I),DECPREC(I))
      END DO

C     Precess the center coordinate.
      DRA =  0.                 ! proper motion of field center
      DDEC = 0.
      CALL NOR_PRECESS(DRA0,DDEC0,DRA,DDEC,
     :                 EQUINOX_CENTER,NEW_EPOCH,DRA0PREC,DDEC0PREC)

C     Clean up and quit.
      WRITE(*,101) NUSE
 101  FORMAT(' Proceeding with ',I4,' positions...')
      RETURN
      END


C
C     Precession subroutine borrowed from Norris spectrograph.
C
      SUBROUTINE NOR_PRECESS(RA,DEC,DRA,DDEC,EPOCH,NEW_EPOCH,
     :                       NEW_RA,NEW_DEC)
C
C     N O R _ P R E C E S S 
C
C     Precession calculations for the Norris Spectrograph.
C
C     Parameters -    (">" input, "<" output)
C
C     (>) RA       (Real*8) Right ascension
C
C     (>) DEC      (Real*8) Declination
C
C     (>) DRA      (Real*4) object proper motion in right ascension in
C                  sec/year
C
C     (>) DDEC     (Real*4) object proper motion in declination in 
C                  arcseconds per year
C
C     (>) EPOCH    (Real*4) Object initial epoch.
C
C     (>) NEW_EPOCH (Real*4) Epoch to which the coordinates are to be
C                  precessed.
C
C     (<) NEW_RA   (Real*8) New right ascension of the object after 
C                  precession.
C
C     (<) NEW_DEC  (Real*8) New declination of object after precession.
C
C     External References:
C
C          ROTATE            Norris Library
C
C     History:   CRO/CIT.  20 June 1988.  Original.
C
      IMPLICIT NONE

      INCLUDE 'math.inc'

C     Formal parameters
      REAL*4 DRA,DDEC,EPOCH,NEW_EPOCH
      REAL*8 RA,DEC,NEW_RA,NEW_DEC
 
C     Local variables
      REAL*4 DYEAR
      REAL*8 DELTARA,DELTADEC
      REAL*8 DT,TAU,Z0,Z,ZJ,XOLD(3),X1(3),X2(3)

C     Proper motion: DEC is in radians; DDEC is in arc-sec/year.
      DYEAR = NEW_EPOCH - EPOCH
      DELTADEC = DYEAR * DDEC      
      DELTADEC = DELTADEC * RPD / 3600.D0
      DEC = DEC + DELTADEC

C     Proper motion: RA is in radians; DRA is in sec of time/year.
      DELTARA = DYEAR * DRA
      DELTARA = DELTARA * 15.D0 * RPD / 3600.D0
      RA = RA + DELTARA

C     Calculate time parameters
      DT  = DBLE((EPOCH - NEW_EPOCH)) * .001
      TAU = DBLE(EPOCH) * 0.001 - 1.900

      Z0 = (0.1117133 + 6.77527E-4 * TAU) * DT +
     :     (1.4656E-4 - 1.31E-6    * TAU) * DT*DT + 8.726E-5 * DT**3

      Z = Z0 + (3.843118E-4 + 3.1998E-6 * TAU)*DT*DT + 1.5514E-6*DT**3

      ZJ = (0.097189874 - 4.13692E-4*TAU - 1.79381E-6*TAU*TAU) * DT - 
     :     (2.0687E-4 + 1.79381E-6*TAU) * DT*DT - 2.02652E-4 * DT**3

      Z = HALFPI + Z
      Z0 = Z0 - HALFPI
      ZJ = -ZJ

C     Precess coordinates using ROTATE().
      XOLD(1) = DCOS(RA) * DCOS(DEC)
      XOLD(2) = DSIN(RA) * DCOS(DEC)
      XOLD(3) = DSIN(DEC)
      CALL ROTATE(3,Z,XOLD,X2)
      CALL ROTATE(1,ZJ,X2,X1)
      CALL ROTATE(3,Z0,X1,X2)

      NEW_DEC = DASIN(X2(3))
      NEW_RA  = DACOS(X2(1)/DCOS(NEW_DEC))

      IF (X2(2) .LT. 0.0) NEW_RA = TWOPI - NEW_RA

      RETURN

      END


C
C     Rotation subroutine borrowed from Don Hamilton of Kitt Peak.
C
      SUBROUTINE ROTATE(M,ALPHA,XOLD,XNEW)
C
C     Rotation subroutine from the Kitt Peak precession program courtesy
C     of Don Hamilton.  Since no documentation came with the code I'm
C     assuming it does what it's supposed to do correctly.  This routine
C     apparently sets up a rotation matrix in S then calls another
C     routine to actually to the matrix multiply.
C
C     Parameters:      ("<" = output, ">" = input)
C
C     (>) M       (Integer*4) Apparently controls the dimension of the
C                 rotation.
C
C     (>) ALPHA   (Real*8) Angle of rotation in radians.
C
C     (>) XOLD    (Real*8 array(3)) Input vector to be rotated.
C
C     (<) XNEW    (Real*8 array(3)) Output vector.
C
C     External References:
C
C          TRNSFM              Norris Library
C
      IMPLICIT NONE

C     Formal parameters
      REAL*8    XOLD(3),XNEW(3),ALPHA
      INTEGER*4 M

C     Local variables
      REAL*8    S(3,3)          ! rotation matrix
      INTEGER*4 I,J

C     Setup rotation matrix.
      DO I=1,3
         DO J=1,3
            IF ((I .NE. M).AND.(J .NE. M)) THEN
               IF (I .NE. J) THEN
                  IF (I .GT. J) S(I,J) = (-1.)*DSIN(ALPHA)
                  IF (I .LT. J) S(I,J) = DSIN(ALPHA)
               ELSE 
                  S(I,J) = DCOS(ALPHA)
               END IF
            ELSE
               S(I,J) = 0.0
               IF (I .EQ. J) S(I,J)=1.0
            END IF
         END DO
      END DO 

C     Rotate vector.
      CALL TRNSFM(S,XOLD,XNEW)

      RETURN

      END 


C
C     Vector multiply subroutine borrowed from Don Hamilton of Kitt Peak.
C
      SUBROUTINE TRNSFM(S,XOLD,XNEW)
C
C     This routine is part of the precession code from Kitt Peak, courtesy
C     of Don Hamilton.  Apparently this is a simple multiplication of 
C     a vector by a matrix.
C 
C     Parameters:            ("<" = output, ">" = input)
C
C     (>) S     (Real*8 Array(3,3)) Transformation matrix.
C
C     (>) XOLD  (Real*8 Array(3)) Input vector.
C
C     (<) XNEW  (Real*8 Array(3)) Output vector.
C
C     No external references.
C
      IMPLICIT NONE

C     Formal parameters
      REAL*8 S(3,3),XOLD(3),XNEW(3)

C     Local variables
      INTEGER*4 I,J

      DO I=1,3
         XNEW(I)=0.0
      END DO

C     Multiply matrix by vector.
      DO I = 1,3
         DO J = 1,3
            XNEW(I) = XNEW(I) + S(I,J)*XOLD(J)
         END DO
      END DO

      RETURN

      END 
