C
C Implement the touchup fix described in the report:
C
C   The Slitmask Astrometry of the LRIS - J.G.Cohen and P.L.Shopbell,
C   December 1996
C
C J.Cohen / Caltech / June 1997
C
      SUBROUTINE SLIT_ASTROMETRY(XIN, YIN, XOUT, YOUT)
C
C Variables:
C
C xin, yin           : the position in mm of the location in the LRIS
C                      slitmask based on the baseline astrometry model.
C xout, yout         : corrected for the touchup fix described in the
C                      above report (in mm)
C xccd, yccd         : pixels on the CCD corresponding to xin, yin
C xccd_out, yccd_out : pixels on the CCD correspoding to xout, yout
C
C
C The fit used is of the form below:
C
C   Xcor = A(2)*x + A(3)*y + A(4)*x*x + A(5)*y*y + A(6)*x*y
C   Ycor = B(2)*x + B(3)*y + B(4)*x*x + B(5)*y*y + B(6)*x*y
C
C Here Xcor, Ycor and x, y are in pixels on the CCD.
C
C Aug-13-97 - Fixed XOUT/YOUT calculations to use XCCD_OUT/YCCD_OUT. [PLS]
C Aug-14-97 - Improved coefficients of touchup fix. [JLC]
C

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4  XIN, YIN, XOUT, YOUT

C     Local variables
      REAL*4  A(6), B(3), XCCD, YCCD, XCCD_OUT, YCCD_OUT

C     Constants for the original touchup fix of Sept 1996
C     DATA A / 0.0, 0.99474994, 0.00840260, 0.45503366E-06,
C    :         -0.40952965E-05, 0.97639775E-06 /
C     DATA B / 0.0, -0.00114755, 0.99906049 /
C
C     Constants for the de-rotated, 0 mean translation touchup fix
C       of August 1997 from the Sep 1996 data
C     Note that here the A(1) and B(1) terms must be used
C
      DATA A / 2.0,  0.99476227, 0.00728125, 0.45412825E-06,
     :         -0.40955557E-05,  0.96690643E-06 /
      DATA B / 1.0, -0.00001856, 0.99906324  /

C
C     Convert mm to CCD pixels. (Approximate, but as long as you
C     use exactly the same formula for inversion, that is OK.)
      XCCD = CCD_SIDE / 2.0 + (X_CENTER - XIN) / CCD_SCALE
      YCCD = CCD_SIDE / 2.0 - (Y_CENTER - YIN) / CCD_SCALE

      XCCD_OUT = A(1) + A(2) * XCCD + A(3) * YCCD + A(4) * XCCD * XCCD
     :         + A(5) * YCCD * YCCD + A(6) * XCCD * YCCD
      YCCD_OUT = B(1) + B(2) * XCCD + B(3) * YCCD

C
C     Now convert back to mm on the slitmask.
C
      XOUT = X_CENTER - (XCCD_OUT - CCD_SIDE / 2.0) * CCD_SCALE
      YOUT = Y_CENTER + (YCCD_OUT - CCD_SIDE / 2.0) * CCD_SCALE

      RETURN
      END
