C
C     Add a slit on the specified object.
C     Return .TRUE. if slit successfully added, else .FALSE.
C     Assume OBJ_STYPE(OBJ) already set to desired slit type.
C
C     Sep-1997 - Fixed bug in SLIT_OK() call.            [PLS]

      LOGICAL FUNCTION ADD_SLIT(OBJ,THETA,SLIT_LEN,SLIT_SEP,
     :                          MIRROR_USE,ENG_TEST)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      INTEGER*4 OBJ
      REAL*8    THETA
      REAL*4    SLIT_LEN,SLIT_SEP
      LOGICAL   MIRROR_USE,ENG_TEST

C     Local variables
      INTEGER*4 STYPE,SIND,ISEG,J
      REAL*4    X1,Y1,Y2,TY1,TY2
      REAL*8    ANGLE,PA1,PA2

C     Functions
      LOGICAL   OBJ_OK,SLIT_OK

C     Check object.
      IF ((.NOT. OBJ_OK(OBJ)) .OR. (NSLITS .EQ. MAXSLITS)) THEN
         ADD_SLIT = .FALSE.
         RETURN
      END IF

C     Determine corresponding slit type and slit index.
      STYPE = OBJ_STYPE(OBJ)
      SIND = OBJ_SIND(OBJ)

C     Compute extent of slit, based on slit type.
      IF (STYPE .EQ. STD_SLIT) THEN
C        Standard slit.
         X1 = OBJ_T_X(OBJ)
         Y1 = OBJ_T_Y(OBJ) - (0.5 * SLIT_LEN)
         Y2 = OBJ_T_Y(OBJ) + (0.5 * SLIT_LEN)
      ELSE IF (STYPE .EQ. TILT_SLIT) THEN
C        Tilted slit.
         ANGLE = TILT(SIND) + (THETA * RPD)
         X1 = OBJ_T_X(OBJ)
         IF (ANGLE .LE. -HALFPI .OR. ANGLE .GE. HALFPI) THEN
            Y1 = OBJ_T_Y(OBJ) + (TILT_MIN_UP(SIND) * DCOS(ANGLE))
            Y2 = OBJ_T_Y(OBJ) - (TILT_MIN_DOWN(SIND) * DCOS(ANGLE))
         ELSE
            Y1 = OBJ_T_Y(OBJ) - (TILT_MIN_DOWN(SIND) * DCOS(ANGLE))
            Y2 = OBJ_T_Y(OBJ) + (TILT_MIN_UP(SIND) * DCOS(ANGLE))
         END IF
      ELSE IF (STYPE .EQ. ARC_SLIT) THEN
C        Arc slit.
         X1 = OBJ_T_X(OBJ)
         PA1 = ARC_MIN_DOWN(SIND) + (THETA * RPD)
         PA2 = ARC_MIN_UP(SIND) + (THETA * RPD)
         TY1 = ARC_RADIUS(SIND) * DCOS(PA1)
         TY2 = ARC_RADIUS(SIND) * DCOS(PA2)
         IF ((PA1 .LE. -PI .AND. PA2 .GE. -PI) .OR.
     :       (PA1 .LE. +PI .AND. PA2 .GE. +PI)) THEN     ! slit crosses +-180
            Y1 = OBJ_T_Y(OBJ) - ARC_RADIUS(SIND)
            IF (TY1 .GT. TY2) THEN
               Y2 = OBJ_T_Y(OBJ) + TY1
            ELSE
               Y2 = OBJ_T_Y(OBJ) + TY2
            END IF
         ELSE IF (PA1 .LE. 0 .AND. PA2 .GE. 0) THEN     ! slit crosses +-0
            IF (TY1 .LT. TY2) THEN
               Y1 = OBJ_T_Y(OBJ) + TY1
            ELSE
               Y1 = OBJ_T_Y(OBJ) + TY2
            END IF
            Y2 = OBJ_T_Y(OBJ) + ARC_RADIUS(SIND)
         ELSE
            IF (TY1 .LT. TY2) THEN
               Y1 = OBJ_T_Y(OBJ) + TY1
               Y2 = OBJ_T_Y(OBJ) + TY2
            ELSE
               Y1 = OBJ_T_Y(OBJ) + TY2
               Y2 = OBJ_T_Y(OBJ) + TY1
            END IF
         END IF
      ELSE IF (STYPE .EQ. LINE_SLIT) THEN
C        Line segment slit.
C        Determine initial segment object.
CCC ??? What if segment objects marked as out of mask ???
         ISEG = OBJ + LINE_INIT(SIND) - SIND
         X1 = OBJ_T_X(ISEG)
         Y1 = OBJ_T_Y(ISEG)
         Y2 = OBJ_T_Y(ISEG)
C        Step through segment objects.
         DO J=2,LINE_NUM(OBJ_SIND(ISEG))
            IF (OBJ_T_Y(ISEG+J-1) .LT. Y1) Y1 = OBJ_T_Y(ISEG+J-1)
            IF (OBJ_T_Y(ISEG+J-1) .GT. Y2) Y2 = OBJ_T_Y(ISEG+J-1)
         END DO
      ELSE IF (STYPE .EQ. BOX_SLIT) THEN
C        Box slit.
         X1 = OBJ_T_X(OBJ)
         Y1 = OBJ_T_Y(OBJ) - (BOX_SIZE(SIND)/2.0)
         Y2 = OBJ_T_Y(OBJ) + (BOX_SIZE(SIND)/2.0)
      END IF

C     Remove overlap with top and bottom edges of mask.
      IF (Y1 .LT. (Y_CENTER - YSIDE/2.0)) Y1 = Y_CENTER - YSIDE/2.0 + 1
      IF (Y2 .GT. (Y_CENTER + YSIDE/2.0)) Y2 = Y_CENTER + YSIDE/2.0 - 1

C     Check for overlap with other assigned slits.
      IF (.NOT.
     :      SLIT_OK(NSLITS+1,STYPE,X1,Y1,Y2,
     :              SLIT_SEP,MIRROR_USE,ENG_TEST)) THEN
         ADD_SLIT = .FALSE.
         RETURN
      END IF

C     Add slit.
      NSLITS = NSLITS + 1
      SLIT_OBJ(NSLITS) = OBJ
      OBJ_SLIT(OBJ) = NSLITS
      OBJ_HIT(OBJ) = .TRUE.
      SLIT_X(NSLITS) = X1
      SLIT_MIN_Y(NSLITS) = Y1
      SLIT_MAX_Y(NSLITS) = Y2

C     Increment counter based on slit type.
C     Tilted slit
      IF (STYPE .EQ. TILT_SLIT) THEN
         MTILT = MTILT + 1
C     Arc slit
      ELSE IF (STYPE .EQ. ARC_SLIT) THEN
         MARCS = MARCS + 1
C     Box slit
      ELSE IF (STYPE .EQ. BOX_SLIT) THEN
         MBOX = MBOX + 1
C     Line segment slit
      ELSE IF (STYPE .EQ. LINE_SLIT) THEN
         MSEGS = MSEGS + 1

C        Also, mark all portions of a line segment slit
         SLIT_OBJ(NSLITS) = ISEG
         DO J=1,LINE_NUM(OBJ_SIND(ISEG))
            OBJ_SLIT(ISEG+J-1) = NSLITS
            OBJ_HIT(ISEG+J-1) = .TRUE.
         END DO
      END IF
      MSPECIAL = MTILT + MARCS + MBOX + MSEGS

      ADD_SLIT = .TRUE.
      RETURN
      END


C
C     Delete a slit on the specified object.
C     Return .TRUE. if slit successfully deleted, else .FALSE.
C
      LOGICAL FUNCTION DELETE_SLIT(OBJ)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      INTEGER OBJ

C     Local variables
      INTEGER*4 SLIT,STYPE,SIND,ISEG,J

C     Check object.
      IF ((.NOT. OBJ_HIT(OBJ)) .OR. (NSLITS .EQ. 0)) THEN
         DELETE_SLIT = .FALSE.
         RETURN
      END IF

C     Determine corresponding slit, slit type, and slit index.
      SLIT = OBJ_SLIT(OBJ)
      STYPE = OBJ_STYPE(OBJ)
      SIND = OBJ_SIND(OBJ)

C     Delete slit.
      OBJ_SLIT(OBJ) = 0
      OBJ_HIT(OBJ) = .FALSE.

C     Decrement counter based on slit type.
C     Tilted slit
      IF (STYPE .EQ. TILT_SLIT) THEN
         MTILT = MTILT - 1
C     Arc slit
      ELSE IF (STYPE .EQ. ARC_SLIT) THEN
         MARCS = MARCS - 1
C     Box slit
      ELSE IF (STYPE .EQ. BOX_SLIT) THEN
         MBOX = MBOX - 1
C     Line segment slit
      ELSE IF (STYPE .EQ. LINE_SLIT) THEN
         MSEGS = MSEGS - 1

C        Also, mark all portions of a line segment slit
         ISEG = OBJ + LINE_INIT(SIND) - SIND    ! initial segment object
         DO J=1,LINE_NUM(OBJ_SIND(ISEG))
            OBJ_SLIT(ISEG+J-1) = 0
            OBJ_HIT(ISEG+J-1) = .FALSE.
         END DO
      END IF
      MSPECIAL = MTILT + MARCS + MBOX + MSEGS

C     Replace deleted slit with final slit entry.
      IF (SLIT .LT. NSLITS) THEN
         SLIT_OBJ(SLIT) = SLIT_OBJ(NSLITS)
         OBJ_SLIT(SLIT_OBJ(SLIT)) = SLIT
         SLIT_X(SLIT) = SLIT_X(NSLITS)
         SLIT_MIN_Y(SLIT) = SLIT_MIN_Y(NSLITS) 
         SLIT_MAX_Y(SLIT) = SLIT_MAX_Y(NSLITS)
      END IF
      NSLITS = NSLITS - 1

      DELETE_SLIT = .TRUE.
      RETURN
      END


C
C     Return nearest object to mask position (X,Y).
C
      INTEGER FUNCTION NEAREST_OBJECT(X,Y)

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    X,Y

C     Local variables
      REAL*4    D,DMIN
      INTEGER*4 I

      NEAREST_OBJECT = 1
      DMIN = (X - OBJ_T_X(1))**2 + (Y - OBJ_T_Y(1))**2

      DO I=2,NPTS
         D = (X - OBJ_T_X(I))**2 + (Y - OBJ_T_Y(I))**2
         IF (D .LT. DMIN) THEN
             DMIN = D
             NEAREST_OBJECT = I
         END IF
      END DO

      RETURN
      END


C
C     Return nearest slit to mask position (X,Y).
C
      INTEGER FUNCTION NEAREST_SLIT(X,Y)

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    X,Y

C     Local variables
      REAL*4    D,DMIN
      INTEGER*4 I

C     If no slits defined, return 0.
      IF (NSLITS .EQ. 0) THEN
         NEAREST_SLIT = 0
         RETURN
      END IF

      NEAREST_SLIT = 1
      DMIN = (X-OBJ_T_X(SLIT_OBJ(1)))**2 + (Y-OBJ_T_Y(SLIT_OBJ(1)))**2

      DO I=2,NSLITS
         D = (X-OBJ_T_X(SLIT_OBJ(I)))**2 + (Y-OBJ_T_Y(SLIT_OBJ(I)))**2 
         IF (D .LT. DMIN) THEN
            DMIN = D
            NEAREST_SLIT = I
         END IF
      END DO

      RETURN
      END


C
C     Return .TRUE. if an object is able to have a slit on it.
C
      LOGICAL FUNCTION OBJ_OK(OBJ)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      INTEGER*4 OBJ

C     Object is OK if it does not already have a slit on it,
C              and if the object is in the field.
      OBJ_OK = ((.NOT. OBJ_HIT(OBJ))  .AND. .NOT. OBJ_OUT(OBJ))

      RETURN
      END


C
C     Return .TRUE. if a slit does not overlap any other slits.
C     Assumes slits are defined as numbers 1...SLIT-1.
C
C     Modifications:
C
C     Aug-1997 - Allow two box slits to overlap.                  [PLS]
C     Aug-1997 - Changed SLIT_SEP from integer to real.           [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.              [PLS]

      LOGICAL FUNCTION SLIT_OK(SLIT,TYPE,X,MINY,MAXY,
     :                         SEP,MIRROR_USE,ENG_TEST)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      INTEGER*4 SLIT,TYPE
      REAL*4    X,MINY,MAXY,SEP
      LOGICAL   MIRROR_USE,ENG_TEST

C     Local variables
      INTEGER*4 K

      SLIT_OK = .TRUE.

C     Slit is OK if not occulted by pickoff mirror.
      IF (MIRROR_USE) THEN
         IF ((   X .GT. GXMIN .AND.    X .LT. GXMAX) .AND.
     :      ((MINY .GT. GYMIN .AND. MINY .LT. GYMAX) .OR.
     :       (MAXY .GT. GYMIN .AND. MAXY .LT. GYMAX))) THEN
            SLIT_OK = .FALSE.
            RETURN
         END IF
      END IF

C     Slit is OK if not occulted by central slitmask bar.
      IF ((MINY .GT. BYMIN .AND. MINY .LT. BYMAX) .OR.
     :    (MAXY .GT. BYMIN .AND. MAXY .LT. BYMAX)) THEN
         SLIT_OK = .FALSE.
         RETURN
      END IF

C     Slit is OK if it's the only one defined.
      IF (SLIT .EQ. 1) RETURN

C     Slit is OK if it does not overlap other assigned slits.
C     Check for four things:
C        1) top of new slit within SEP of another slit
C        2) bottom of new slit within SEP of another slit
C        3) new slit surrounds another slit (needed for non-std slits)
C        4) slits are not both boxes (in which case overlap is ok)
      DO K=1,SLIT-1
         IF (((MINY .GT. SLIT_MIN_Y(K) - SEP .AND.        ! bottom within slit
     :         MINY .LT. SLIT_MAX_Y(K) + SEP) .OR.        !      OR
     :        (MAXY .GT. SLIT_MIN_Y(K) - SEP .AND.        ! top within slit
     :         MAXY .LT. SLIT_MAX_Y(K) + SEP) .OR.        !      OR
     :        (MINY .LT. SLIT_MIN_Y(K) .AND.              ! surrounds slit
     :         MAXY .GT. SLIT_MAX_Y(K))) .AND.            !      AND
     :       (TYPE .NE. BOX_SLIT .OR.                     ! not both boxes
     :        OBJ_STYPE(SLIT_OBJ(K)) .NE. BOX_SLIT)) THEN

            SLIT_OK = .FALSE.

C           It's ok for slits to overlap in engineering mode.
            IF (ENG_TEST) THEN
               SLIT_OK = .TRUE.
               WRITE(*,*)''
               WRITE(*,101)
     :               ' *******************************************'
               WRITE(*,101)' Engineering test mode selected.'
               WRITE(*,101)' Overlapping slit ignored.'
               WRITE(*,101)
     :               ' *******************************************'
 101           FORMAT(A)
            END IF
         END IF
      END DO

      RETURN
      END


C
C     Return object with specified name.
C
      INTEGER FUNCTION LOCATE_OBJECT (NAME)

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      CHARACTER NAME*40

C     Local variables
      INTEGER*4 NLEN,I

C     Compute length of name string
      NLEN = 1
      DO WHILE (NAME(NLEN:NLEN) .NE. ' ')
         NLEN = NLEN + 1
      END DO

C     Locate object with matching name
      I = 1
      DO WHILE ((I .LE. NPTS) .AND.
     :          (NAME(1:NLEN) .NE. ONAMES(I)(1:NLEN)))
         I = I + 1
      END DO

      IF (I .GT. NPTS) THEN
         LOCATE_OBJECT = 0
      ELSE
         LOCATE_OBJECT = I
      END IF

      RETURN
      END


C
C     Lengthen slits so each slit covers the maximum sky possible.
C
C     Sep-1997 - Allowed tilted slits to be lengthened.              [PLS]
C     Oct-1997 - Fixed major bug (typo) in lengthening bottom slit.  [PLS]
C     Jan-1998 - Fixed minor bug (typo) in non-std slits near bar.   [PLS]
C
C     BUG: There is a slight bug in here.  Tilted slits may be
C          lengthened such that they extend off the mask in X.
C          This should be fixed at some time.  In the meantime,
C          solution is to add a dummy regular slit to prevent the
C          tilted slit from extending too far.  This is primarily
C          a problem if the topmost or bottommost slit is tilted.

      SUBROUTINE LENGTHEN_SLITS(SEP)

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4    SEP

C     Local variables
      INTEGER*4 STYPE1,STYPE2,I
      INTEGER*4 YY(ENG_SLITS)                    ! hack to fix broken ISORT()
      REAL*4    AVG_Y

      WRITE(*,*)''
      WRITE(*,11)' Maximizing slit length...'
 11   FORMAT(A)

C     Check for null case of 0 slits.
      IF (NSLITS .EQ. 0) THEN
         CALL USER_ERR("No slits assigned.",18)
         RETURN
      END IF

C     Returns array INDX such that SLIT_MIN_Y(INDX(I))
C       is in decreasing order.
      DO I = 1,NSLITS
         YY(I) = SLIT_MIN_Y(I) + (YSIDE / 2.0)    ! hack to fix broken ISORT()
      END DO
      CALL ISORT(NSLITS,YY,INDX)

C     Debugging...
C     DO I = 1,NSLITS
C        WRITE(*,*) SLIT_MIN_Y(INDX(I)),'--',SLIT_MAX_Y(INDX(I))
C     END DO

C     Place ends of each slit so they are SEP units (mm) apart in Y.
C     Extend standard and tilted slits.
      DO I = 1,NSLITS-1
         STYPE1 = OBJ_STYPE(SLIT_OBJ(INDX(I)))
         STYPE2 = OBJ_STYPE(SLIT_OBJ(INDX(I+1)))

C        Check for presence of sitmask bar between slits.
         IF ((SLIT_MIN_Y(INDX(I)) .GE. BYMAX) .AND.
     :       (SLIT_MAX_Y(INDX(I+1)) .LE. BYMIN)) THEN
            IF ((STYPE1 .EQ. STD_SLIT) .OR.
     :          (STYPE1 .EQ. TILT_SLIT)) THEN
               SLIT_MIN_Y(INDX(I)) = BYMAX
            END IF
            IF ((STYPE2 .EQ. STD_SLIT) .OR.
     :          (STYPE2 .EQ. TILT_SLIT)) THEN
               SLIT_MAX_Y(INDX(I+1)) = BYMIN
            END IF
C        Extend all other standard and tilted slits.
         ELSE IF ((STYPE1 .EQ. STD_SLIT) .OR.
     :            (STYPE1 .EQ. TILT_SLIT)) THEN
            IF ((STYPE2 .EQ. STD_SLIT) .OR.
     :          (STYPE2 .EQ. TILT_SLIT)) THEN
C              Both slits are standard or tilted: extend both.
               AVG_Y = (SLIT_MIN_Y(INDX(I)) + SLIT_MAX_Y(INDX(I+1)))/2.0
               SLIT_MIN_Y(INDX(I))   = AVG_Y + (SEP / 2.0)
               SLIT_MAX_Y(INDX(I+1)) = AVG_Y - (SEP / 2.0)
            ELSE
C              Extend only slit I.
               SLIT_MIN_Y(INDX(I))   = SLIT_MAX_Y(INDX(I+1)) + SEP
            END IF
         ELSE
            IF ((STYPE2 .EQ. STD_SLIT) .OR.
     :          (STYPE2 .EQ. TILT_SLIT)) THEN
C              Extend only slit I+1.
               SLIT_MAX_Y(INDX(I+1)) = SLIT_MIN_Y(INDX(I)) - SEP
            END IF
         END IF
      END DO

C     Place ends of extreme Y slits at the edge of the mask.
      IF ((OBJ_STYPE(SLIT_OBJ(INDX(1))) .EQ. STD_SLIT) .OR.
     :    (OBJ_STYPE(SLIT_OBJ(INDX(1))) .EQ. TILT_SLIT)) THEN
         IF (SLIT_MAX_Y(INDX(1)) .LT. BYMAX) THEN
            SLIT_MAX_Y(INDX(1))      = BYMIN
         ELSE
            SLIT_MAX_Y(INDX(1))      = Y_CENTER + (YSIDE / 2.0)
         END IF
      END IF
      IF ((OBJ_STYPE(SLIT_OBJ(INDX(NSLITS))) .EQ. STD_SLIT) .OR.
     :    (OBJ_STYPE(SLIT_OBJ(INDX(NSLITS))) .EQ. TILT_SLIT)) THEN
         IF (SLIT_MIN_Y(INDX(NSLITS)) .GT. BYMIN) THEN
            SLIT_MIN_Y(INDX(NSLITS)) = BYMAX
         ELSE
            SLIT_MIN_Y(INDX(NSLITS)) = Y_CENTER - (YSIDE / 2.0)
         END IF
      END IF

      RETURN
      END
