C
C     Create output file 3, the mill control file for creating the slitmask.
C     
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.         [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.              [PLS]
C     Oct-1997 - Fixed bug in field center for southern fields.   [PLS]

      SUBROUTINE WRITE_MILL (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                       RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                       SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                       LAT, POS_ANGLE, H, THETA, MIRROR_USE,
     :                       ENG_TEST, INTERACTIVE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      CHARACTER INPUT*40,OUTPUT*40,OUT2*40,OUT3*40
      INTEGER*4 PRIORITY,RANK_TOP,RANK_BOT
      REAL*4    NEW_EPOCH
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    TEMP,PRES,WAVE,LAT,POS_ANGLE,H,THETA
      LOGICAL   MIRROR_USE,ENG_TEST,INTERACTIVE

C     Local variables
      CHARACTER TODAY*24,USERID*8,USERNAME*20,HOSTNAME*24
      INTEGER*4 HRRA,MNRA,DGDEC,MNDEC
      INTEGER*4 I,IOS,OLUN
      REAL*8    RA2,SCRA,DEC2,SCDEC
      REAL*4    SLIT_WID_MM
      LOGICAL   VERBOSE

      WRITE(*,*)''
      WRITE(*,11)' Writing mill control file...'
 11   FORMAT(A)

C     Open output mill control file.
      OLUN = 20
      OPEN(OLUN,FILE=OUT3,FORM='FORMATTED',STATUS='UNKNOWN',
     :     IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         WRITE(*,30) OUT3
 30      FORMAT(' Unable to open output file: ',A)
         STOP 'Aborting...'
      END IF

C     Output header information.
      WRITE(OLUN,39)
     :      LDL3,'                                               ',RDL3
 39   FORMAT(A1,A,A1)
      WRITE(OLUN,40) LDL3, VERSION, RDL3
 40   FORMAT(A1,
     :       '  ****       AUTOSLIT  ',F3.1,' OUTPUT        ****  ',A1)
      CALL FDATE(TODAY)
      WRITE(OLUN,41) LDL3, TODAY, RDL3
 41   FORMAT(A1, '  ****     ',A24,'      ****  ', A1)
      WRITE(OLUN,42) LDL3, RDL3
 42   FORMAT(A1, '  ****                                   ****  ', A1)
      CALL GET_USER (USERID, 8, USERNAME, 20)
      WRITE(OLUN,43) LDL3, USERNAME, USERID, RDL3
 43   FORMAT(A1, '  ****  ',A20,' (',A8,')  ****  ', A1)
      CALL GET_HOST (HOSTNAME, 24)
      WRITE(OLUN,45) LDL3, HOSTNAME, RDL3
 45   FORMAT(A1, '  ****     ',A24,'      ****  ', A1)
      WRITE(OLUN,47)
     :      LDL3,'                                               ',RDL3
 47   FORMAT(A1,A,A1)

C     Output summary of physical parameters.
      VERBOSE = .FALSE.
      CALL PRINT_PARAMS (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                   RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                   SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                   LAT, POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                   INTERACTIVE, VERBOSE, LDL3, RDL3, OLUN)

C     Prior to April 1995, the output was the centerfield coordinates
C     precessed and corrected for refraction (RA0 and DEC0).  After,
C     it is the centerfield coordinates corrected for precession only
C     (DAR0PREC,DDEC0PREC).

      RA2 = DRA0PREC/RPH
      HRRA = INT(RA2)
      MNRA = INT((RA2-HRRA)*60.0)
      SCRA = ((RA2-HRRA)*60.0-MNRA)*60.0
      DEC2 = DDEC0PREC/RPD
      DGDEC = INT(DEC2)
      MNDEC = INT((DEC2-DGDEC)*60.0)
      SCDEC = ((DEC2-DGDEC)*60.0 - MNDEC)*60.0
      IF (DEC2 .LT. 0) THEN
         MNDEC = -MNDEC
         SCDEC = -SCDEC
      END IF

C     Output center of field.
      WRITE(OLUN,51)
     :  LDL3,'                                               ',RDL3
      WRITE(OLUN,51)
     :  LDL3,'  ===========================================  ',RDL3
      WRITE(OLUN,51)
     :  LDL3,'                Center of Field                ',RDL3
      WRITE(OLUN,51)
     :  LDL3,'  -------------------------------------------  ',RDL3
      WRITE(OLUN,51)
     :  LDL3,'  Center of Field is:                          ',RDL3
 51   FORMAT(A1,A,A1)
      WRITE(OLUN,61) LDL3,HRRA,MNRA,SCRA,NEW_EPOCH,RDL3
 61   FORMAT(
     :   A1,'  RA  = ',I4,'h ',I4,'m ',F7.3,'s    [',F7.2,']      ',A1)
      WRITE(OLUN,63) LDL3,DGDEC,MNDEC,SCDEC,RDL3
 63   FORMAT(
     :   A1,'  DEC = ',i4,'d ',i4,'m ',F7.3,'s                   ',A1)
      WRITE(OLUN,65)
     :  LDL3,'  ===========================================  ',RDL3
 65   FORMAT(A1,A,A1)

C     Draw small circular "slit" if using pickoff mirror.
      IF (MIRROR_USE) THEN
         WRITE(OLUN,69) MILL_LABEL
 69      FORMAT(AT)
         CALL MILL_PICKOFF(OLUN,ENG_TEST)
      END IF

C     Output slit milling commands.
      SLIT_WID_MM = SLIT_WID * SCALE
      DO I = 1,NSLITS
         WRITE(OLUN,71) MILL_LABEL
         CALL MILL_SLIT(I,SLIT_WID_MM,OLUN,THETA,ENG_TEST)
      END DO
      WRITE(OLUN,71) MILL_LABEL           ! Final label marks end of file.
 71   FORMAT(A)

C     Clean up and quit.
      WRITE(*,101) OUT3
 101  FORMAT(' File written: ',A)
      CLOSE(OLUN)
      RETURN

      END


C
C     Output circular "slit" marking the pickoff mirror to
C     mill control file.
C     Include astrometric touchup correction if necessary.
C     Note: Mill requires that slits be drawn in counterclockwise
C           direction (in both slitmask and mill coordinate frames).
C
      SUBROUTINE MILL_PICKOFF(OLUN,ENG_TEST)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      INTEGER   OLUN
      LOGICAL   ENG_TEST

C     Local variables
      INTEGER*4 I,NCIRC
      REAL*4    XMILL,YMILL,ZMILL
      REAL*4    X(2*MAX_ARC_LEN),Y(2*MAX_ARC_LEN)
      REAL*8    ANGLE,DANGLE
      LOGICAL   ACORR

      ZMILL = 0.0
      ACORR = (.NOT. ENG_TEST)

C     Draw circular slit.
      NCIRC = INT(TWOPI * R_GUIDER) * 2
      DANGLE = TWOPI / (REAL(NCIRC-1))
      ANGLE = 0.
      DO I=1,NCIRC
         X(I) = G_X - (R_GUIDER * DSIN(ANGLE) * CLOCKWISE_MILL)
         Y(I) = G_Y + (R_GUIDER * DCOS(ANGLE))
         ANGLE = ANGLE + DANGLE
      END DO
C     Apply astrometry correction.
      IF (ACORR) THEN
         DO I=1,NCIRC
            CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
         END DO
      END IF
C     Close polygon.
      X(NCIRC+1) = X(1)
      Y(NCIRC+1) = Y(1)
C     Output points.
      DO I=1,NCIRC+1
         CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
         WRITE(OLUN,11) XMILL, YMILL, ZMILL
 11      FORMAT(F12.6,F12.6,F12.6)
      END DO

      RETURN
      END


C
C     Output vertices of a slit to mill control file.
C     Include astrometric touchup correction if necessary.
C     Note: This code is almost identical to that used for
C           drawing a slit using PGPLOT; see PLOT_SLIT().
C     Note: Mill requires that slits be drawn in counterclockwise
C           direction (in both slitmask and mill coordinate frames).
C
C     Sep-1997 - Adjusted for lengthening tilted slits.            [PLS]

      SUBROUTINE MILL_SLIT(SLIT,WIDTH,OLUN,THETA,ENG_TEST)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      INTEGER   SLIT,OLUN
      REAL*4    WIDTH
      REAL*8    THETA
      LOGICAL   ENG_TEST

C     Local variables
      INTEGER*4 OBJ,STYPE,SIND,I,J,N
      REAL*4    X1,X2,RAD,HWIDTH,XMILL,YMILL,ZMILL
      REAL*4    X(2*MAX_ARC_LEN),Y(2*MAX_ARC_LEN)
      REAL*8    ANGLE,DANGLE
      LOGICAL   ACORR

      HWIDTH = 0.5 * WIDTH * CLOCKWISE_MILL
      ZMILL = 0.0
      ACORR = (.NOT. ENG_TEST)

C     Determine corresponding object, slit type, and slit index.
      OBJ = SLIT_OBJ(SLIT)
      STYPE = OBJ_STYPE(OBJ)
      SIND = OBJ_SIND(OBJ)

C     Ouput vertices of slit, based on slit type.
      IF (STYPE .EQ. STD_SLIT) THEN
C        Standard slit.
         X(1) = SLIT_X(SLIT)-HWIDTH
         Y(1) = SLIT_MIN_Y(SLIT)
         X(2) = SLIT_X(SLIT)-HWIDTH
         Y(2) = SLIT_MAX_Y(SLIT)
         X(3) = SLIT_X(SLIT)+HWIDTH
         Y(3) = SLIT_MAX_Y(SLIT)
         X(4) = SLIT_X(SLIT)+HWIDTH
         Y(4) = SLIT_MIN_Y(SLIT)
C        Apply astrometry correction.
         IF (ACORR) THEN
            DO I=1,4
               CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
            END DO
         END IF
C        Close slit polygon.
         X(5) = X(1)
         Y(5) = Y(1)
C        Output slit points.
         DO I=1,5
            CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
            WRITE(OLUN,21) XMILL, YMILL, ZMILL
 21         FORMAT(F12.6,F12.6,F12.6)
         END DO

      ELSE IF (STYPE .EQ. TILT_SLIT) THEN
C        Tilted slit.
C        Note that internal position angle is opposite.
         ANGLE = TILT(SIND) + (THETA * RPD)
         IF (ANGLE .LE. -HALFPI .OR. ANGLE .GE. HALFPI) THEN
            X1 = SLIT_X(SLIT) +
     :           ((SLIT_MIN_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
            X2 = SLIT_X(SLIT) +
     :           ((SLIT_MAX_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
C           This is same calculation, but depends on TILT_MIN_*.
C           X1 = SLIT_X(SLIT) + (TILT_MIN_UP(SIND) * DSIN(ANGLE))
C           X2 = SLIT_X(SLIT) - (TILT_MIN_DOWN(SIND) * DSIN(ANGLE))
        ELSE
            X1 = SLIT_X(SLIT) +
     :           ((SLIT_MIN_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
            X2 = SLIT_X(SLIT) +
     :           ((SLIT_MAX_Y(SLIT) - OBJ_T_Y(OBJ)) * DTAN(ANGLE))
C           This is same calculation, but depends on TILT_MIN_*.
C           X1 = SLIT_X(SLIT) - (TILT_MIN_DOWN(SIND) * DSIN(ANGLE))
C           X2 = SLIT_X(SLIT) + (TILT_MIN_UP(SIND) * DSIN(ANGLE))
         END IF
         X(1) = X1-HWIDTH
         Y(1) = SLIT_MIN_Y(SLIT)
         X(2) = X2-HWIDTH
         Y(2) = SLIT_MAX_Y(SLIT)
         X(3) = X2+HWIDTH
         Y(3) = SLIT_MAX_Y(SLIT)
         X(4) = X1+HWIDTH
         Y(4) = SLIT_MIN_Y(SLIT)
C        Apply astrometry correction.
         IF (ACORR) THEN
            DO I=1,4
               CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
            END DO
         END IF
C        Close slit polygon.
         X(5) = X(1)
         Y(5) = Y(1)
C        Output slit points.
         DO I=1,5
            CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
            WRITE(OLUN,31) XMILL, YMILL, ZMILL
 31         FORMAT(F12.6,F12.6,F12.6)
         END DO

      ELSE IF (STYPE .EQ. ARC_SLIT) THEN
C        Arc slit.
C        Compute angular extent and number of points.
         RAD = ARC_RADIUS(SIND)
         ANGLE = ARC_MIN_UP(SIND) - ARC_MIN_DOWN(SIND)
         N = INT(ANGLE * RAD) * 2
         DANGLE = ANGLE / (REAL(N-1))
C        Determine point for each angular step.
         ANGLE = ARC_MIN_DOWN(SIND) + (THETA * RPD)
         J = 2*N
         DO I=1,N
            X(I) = OBJ_T_X(OBJ) + (RAD * DSIN(ANGLE)) - HWIDTH
            Y(I) = OBJ_T_Y(OBJ) + (RAD * DCOS(ANGLE))

            X(J) = X(I) + (2.0 * HWIDTH)
            Y(J) = Y(I)

            ANGLE = ANGLE + DANGLE
            J = J - 1
         END DO
C        Apply astrometry correction.
         IF (ACORR) THEN
            DO I=1,2*N
               CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
            END DO
         END IF
C        Close slit polygon.
         X(2*N+1) = X(1)
         Y(2*N+1) = Y(1)
C        Output slit points.
         DO I=1,2*N+1
            CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
            WRITE(OLUN,43) XMILL, YMILL, ZMILL
 43         FORMAT(F12.6,F12.6,F12.6)
         END DO

      ELSE IF (STYPE .EQ. LINE_SLIT) THEN
C        Line segment slit.
         N = LINE_NUM(SIND)
         J = 2*N
C        Check for CW/CCW order of points.
         IF (OBJ_T_Y(OBJ) .GT. OBJ_T_Y(OBJ+1))
     :      HWIDTH = -HWIDTH
         DO I=1,N
            X(I) = OBJ_T_X(OBJ+I-1) - HWIDTH
            Y(I) = OBJ_T_Y(OBJ+I-1)

            X(J) = X(I) + (2.0 * HWIDTH)
            Y(J) = Y(I)
            J = J - 1
         END DO
C        Apply astrometry correction.
         IF (ACORR) THEN
            DO I=1,2*N
               CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
            END DO
         END IF
C        Close slit polygon.
         X(2*N+1) = X(1)
         Y(2*N+1) = Y(1)
C        Output slit points.
         DO I=1,2*N+1
            CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
            WRITE(OLUN,53) XMILL, YMILL, ZMILL
 53         FORMAT(F12.6,F12.6,F12.6)
         END DO

      ELSE IF (STYPE .EQ. BOX_SLIT) THEN
C        Box slit.
         X1 = SLIT_X(SLIT) - (BOX_SIZE(SIND)/2.0)
         X2 = SLIT_X(SLIT) + (BOX_SIZE(SIND)/2.0)
         X(1) = X1
         Y(1) = SLIT_MIN_Y(SLIT)
         X(2) = X1
         Y(2) = SLIT_MAX_Y(SLIT)
         X(3) = X2
         Y(3) = SLIT_MAX_Y(SLIT)
         X(4) = X2
         Y(4) = SLIT_MIN_Y(SLIT)
C        Apply astrometry correction.
         IF (ACORR) THEN
            DO I=1,4
               CALL SLIT_ASTROMETRY (X(I), Y(I), X(I), Y(I))
            END DO
         END IF
C        Close slit polygon.
         X(5) = X(1)
         Y(5) = Y(1)
C        Output slit points.
         DO I=1,5
            CALL MILL_COORDS (X(I), Y(I), XMILL, YMILL)
            WRITE(OLUN,61) XMILL, YMILL, ZMILL
 61         FORMAT(F12.6,F12.6,F12.6)
         END DO
      END IF

      RETURN
      END


C
C     Translate slitmask (x,y) to mill coordinate system.
C     Coordinates reflect:
C       1) Slitmask is flipped around the Y axis and
C          rotated 90 deg CCW before mill operations.
C       2) Center of mask is (X_CENTER,Y_CENTER) in mask coordinates;
C          center of mask is (X_MILL,Y_MILL) in mill coordinates.
C       3) Additional offset of (XOFF_MILL,YOFF_MILL) (for mask holder?).
C
      SUBROUTINE MILL_COORDS (XIN, YIN, XOUT, YOUT)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*4  XIN, YIN, XOUT, YOUT

      XOUT = (Y_CENTER - YIN) + X_MILL + XOFF_MILL
      YOUT = (XIN - X_CENTER) + Y_MILL + YOFF_MILL

      RETURN
      END
