C
C     Create output file 1, which contains a one-line
C     description of each slit in the final mask.
C     
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.         [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.              [PLS]
C     Oct-1997 - Changed COSD/SIND to DCOSD/DSIND.                [PLS]
C     Oct-1997 - Fixed bug in field center for southern fields.   [PLS]

      SUBROUTINE WRITE_SETUP(INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                       RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                       SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                       LAT, POS_ANGLE, H, MIRROR_USE,
     :                       ENG_TEST,INTERACTIVE)

C     The file has the following form:
C
C     The first record of the file begins with a "#" and contains
C     the Input file, the rotation angle of the telescope, the
C     latitude of the observatory, the temperature and pressure
C     of the atmosphere, the wavelengh of light, the epoch precessed
C     to, and the hour angle of the setup.
C
C     Then the guide star information is given, again beginning each
C     line with a "#".   Contained in the list is the name and
C     coordinates of the guidestar, as well as the coordinates of the
C     center of the field.
C
C     Subsequent records are of the following form:
C
C        x_star,y-star,min_y,max_y,prior,name,and mag
C
C     where
C        x_star is the x coordinate of the object and slit in the
C               slit mask frame
C        y_star is the y coordinate of the object in the slit mask
C               frame
C        min_y  is the y coordinate of the beginning of the slit in
C               the slit mask frame
C        max_y  is the y coordinate of the end of the slit in the
C               slit mask frame
C        prior  is the priority of the star in the slit
C        name   is the name of the star in the slit
C        mag    is the magnitude of the star in the slit
C
C     Aug-13-97 - Added DCOS(DECREFR(OBJ)) term to XARC computation.  [PLS]
C     Sep-08-97 - Fixed blank lines in setup file to use LDL1/RDL1.   [PLS]
C

      IMPLICIT  NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      CHARACTER INPUT*40,OUTPUT*40,OUT2*40,OUT3*40
      INTEGER*4 PRIORITY,RANK_TOP,RANK_BOT
      REAL*4    NEW_EPOCH
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    TEMP,PRES,WAVE,LAT,POS_ANGLE,H
      LOGICAL   MIRROR_USE,ENG_TEST,INTERACTIVE

C     Local variables
      INTEGER*4 HRRA,MNRA,DGDEC,MNDEC
      INTEGER*4 IOS,OLUN,OBJ,I
      REAL*8    XARC,YARC,XROT,YROT,XCCD,YCCD
      REAL*8    RA2,SCRA,DEC2,SCDEC
      CHARACTER TODAY*24
      LOGICAL   VERBOSE

      WRITE(*,*)''
      WRITE(*,11)' Writing setup file...'
 11   FORMAT(A)

C     Open output setup file.
      OLUN = 64
      OPEN(OLUN,FILE=OUTPUT,FORM='FORMATTED',STATUS='UNKNOWN',
     :     IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         WRITE(*,30) OUTPUT
 30      FORMAT(' Unable to open output file: ',A)
         WRITE(*,33) DEF_OUTPUT
 33      FORMAT(' Trying: ',A)
         OPEN(OLUN,FILE=OUTPUT,FORM='FORMATTED',STATUS='UNKNOWN',
     :        IOSTAT=IOS)
         IF (IOS .NE. 0) THEN
            WRITE(*,36) DEF_OUTPUT
 36         FORMAT(' Unable to open default  output file: ',A)
            STOP 'Aborting...'
         END IF
      END IF

C     Output header information.
      WRITE(OLUN,39)
     :      LDL1,'                                               ',RDL1
 39   FORMAT(A1,A,A1)
      WRITE(OLUN,40) LDL1, VERSION, RDL1
 40   FORMAT(A1,
     :       '  ****       AUTOSLIT  ',F3.1,' OUTPUT        ****  ',A1)
      CALL FDATE(TODAY)
      WRITE(OLUN,41) LDL1, TODAY, RDL1
 41   FORMAT(A1,'  ****     ',A24,'      ****  ',A1)
      WRITE(OLUN,42)
     :      LDL1,'                                               ',RDL1
 42   FORMAT(A1,A,A1)

C     Output summary of physical parameters.
      VERBOSE = .FALSE.
      CALL PRINT_PARAMS (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                   RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                   SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                   LAT, POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                   INTERACTIVE, VERBOSE, LDL1, RDL1, OLUN)

C     Prior to April 1995, the output was the centerfield coordinates
C     precessed and corrected for refraction (RA0 and DEC0).  After,
C     it is the centerfield coordinates corrected for precession only
C     (DAR0PREC,DDEC0PREC).

      RA2 = DRA0PREC/RPH
      HRRA = INT(RA2)
      MNRA = INT((RA2-HRRA)*60.0)
      SCRA = ((RA2-HRRA)*60.0-MNRA)*60.0
      DEC2 = DDEC0PREC/RPD
      DGDEC = INT(DEC2)
      MNDEC = INT((DEC2-DGDEC)*60.0)
      SCDEC = ((DEC2-DGDEC)*60.0 - MNDEC)*60.0
      IF (DEC2 .LT. 0) THEN
         MNDEC = -MNDEC
         SCDEC = -SCDEC
      END IF

C     Output guide star information.
      IF (MIRROR_USE) THEN
         WRITE(OLUN,51)
     :     LDL1,'                                               ',RDL1
         WRITE(OLUN,51)
     :     LDL1,'  ===========================================  ',RDL1
         WRITE(OLUN,51)
     :     LDL1,'              Guide Star Identity              ',RDL1
         WRITE(OLUN,51)
     :     LDL1,'  -------------------------------------------  ',RDL1
 51      FORMAT(A1,A,A1)
         WRITE(OLUN,54) LDL1,NSPECIAL,RDL1
 54      FORMAT(A1,
     :          '  Guide star is STAR # ',I4,'                    ',A1)
         WRITE(OLUN,55) LDL1,GUIDE_STAR_NAME,RDL1
 55      FORMAT(A1,'  ',A40,'     ',A1)
         WRITE(OLUN,56) LDL1,G_X,G_Y,RDL1
 56      FORMAT(A1,
     :         '  At X = ',F6.1,' Y = ',F6.1,'                     ',A1)
         WRITE(OLUN,57)
     :     LDL1,'  ===========================================  ',RDL1
 57      FORMAT(A1,A,A1)
      END IF

C     Output center of field.
      WRITE(OLUN,61)
     :  LDL1,'                                               ',RDL1
      WRITE(OLUN,61)
     :  LDL1,'  ===========================================  ',RDL1
      WRITE(OLUN,61)
     :  LDL1,'                Center of Field                ',RDL1
      WRITE(OLUN,61)
     :  LDL1,'  -------------------------------------------  ',RDL1
      WRITE(OLUN,61)
     :  LDL1,'  Center of Field is:                          ',RDL1
 61   FORMAT(A1,A,A1)
      WRITE(OLUN,71) LDL1,HRRA,MNRA,SCRA,NEW_EPOCH,RDL1
 71   FORMAT(
     :   A1,'  RA  = ',I4,'h ',I4,'m ',F7.3,'s    [',F7.2,']      ',A1)
      WRITE(OLUN,73) LDL1,DGDEC,MNDEC,SCDEC,RDL1
 73   FORMAT(
     :   A1,'  DEC = ',i4,'d ',i4,'m ',F7.3,'s                   ',A1)
      WRITE(OLUN,75)
     :  LDL1,'  ===========================================  ',RDL1
 75   FORMAT(A1,A,A1)

C     Output slit positions.
      WRITE(OLUN,81)
     :  LDL1,'                                               ',RDL1
      WRITE(OLUN,81)
     :  LDL1,'  ===========================================  ',RDL1
      WRITE(OLUN,81)
     :  LDL1,'                   Slit Setup                  ',RDL1
      WRITE(OLUN,81)
     :  LDL1,'  -------------------------------------------  ',RDL1
 81   FORMAT(A1,A,A1)
      WRITE(OLUN,93) LDL1,'  X_STAR  ','  Y_STAR  ','   MIN_Y  ',
     :               '   MAX_Y  ',' Percent',' CCD_X',' CCD_Y','Prior',
     :               '  Name        ','  Mag  ',RDL1
 93   FORMAT(A1,A10,T12,A10,T24,A10,T36,A10,T47,A8,
     :       T57,A6,T65,A6,T72,A5,T80,A14,T95,A7,A1)
      DO I=1,NSLITS
         OBJ = SLIT_OBJ(I)
C        Compute CCD coordinates.
         XARC = (RAREFR(OBJ) - DRA0REFR) * DCOS(DECREFR(OBJ)) / RPAS
         YARC = (DECREFR(OBJ) - DDEC0REFR) / RPAS
         XROT = DCOSD(POS_ANGLE) * XARC - DSIND(POS_ANGLE) * YARC
         YROT = DSIND(POS_ANGLE) * XARC + DCOSD(POS_ANGLE) * YARC
         CALL CCD_COORD_PREDICT (XROT, YROT, XCCD, YCCD)
C        Output slit setup.
         WRITE(OLUN,95) X_CENTER + (XS_SLIT/2.0) - SLIT_X(I),
     :                  (YS_SLIT/2.0) - OBJ_T_Y(OBJ) - Y_CENTER,
     :                  (YS_SLIT/2.0) - SLIT_MIN_Y(I) - Y_CENTER,
     :                  (YS_SLIT/2.0) - SLIT_MAX_Y(I) - Y_CENTER,
     :                  (OBJ_T_Y(OBJ) - SLIT_MIN_Y(I))/
     :                    (SLIT_MAX_Y(I) - SLIT_MIN_Y(I)) * 100.0,
     :                  XCCD, YCCD, PRIORS(OBJ), ONAMES(OBJ), MAGS(OBJ)
 95      FORMAT(' ',F10.5,T12,F10.5,T24,F10.5,T36,F10.5,T47,F8.4,
     :          T57,F6.1,T65,F6.1,T73,I4,T80,A14,T95,F7.3)
      END DO
      WRITE(OLUN,99)
     :  LDL1,'  ===========================================  ',RDL1
 99   FORMAT(A1,A,A1)

C     Clean up and quit.
      WRITE(*,201) OUTPUT
 201  FORMAT(' File written: ',A)
      CLOSE (OLUN)
      RETURN

      END
